from pathlib import PurePosixPath
from dbxdeploy.git.CurrentBranchResolver import CurrentBranchResolver


class WorkspaceBaseDirFactory:

    current_branch_placeholder = "{current_branch}"

    def __init__(
        self,
        workspace_base_dir_template: str,
        current_branch_resolver: CurrentBranchResolver,
    ):
        self.__workspace_base_dir_template = workspace_base_dir_template
        self.__current_branch_resolver = current_branch_resolver

    def create(self):
        if self.current_branch_placeholder not in self.__workspace_base_dir_template:
            return PurePosixPath(self.__workspace_base_dir_template)

        current_git_branch = self.__current_branch_resolver.resolve()

        return PurePosixPath(self.__workspace_base_dir_template.replace(self.current_branch_placeholder, current_git_branch))



from typing import Iterable
from pathlib import Path
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.package.Dependency import Dependency


class LocalPathsResolver:
    def __init__(
        self,
        project_base_dir: Path,
    ):
        self.__project_base_dir = project_base_dir

    def get_package_dist_path(self, package_metadata: PackageMetadata):
        dist_dir = self.__project_base_dir.joinpath("dist")

        return dist_dir.joinpath(package_metadata.get_package_filename())

    def get_dependency_dist_path(self, dependency: Dependency):
        dist_dir = self.__project_base_dir.joinpath("dist")

        return self.get_dependency_path_from_dir(dist_dir, dependency)

    def get_dependency_path_from_dir(self, directory: Path, dependency: Dependency):
        dependency_paths = self.__get_all_dependency_paths_from_dir(directory)
        dependency_path = self.__find_dependency_path(dependency_paths, dependency)

        if dependency_path:
            return dependency_path

        raise Exception(f"Dependency {dependency.dependency_name}=={dependency.dependency_version} not found in {directory} directory")

    def get_linux_dependency_path_from_dir(self, directory: Path, dependency: Dependency):
        dependency_paths = self.__get_linux_dependency_paths_from_dir(directory)
        dependency_path = self.__find_dependency_path(dependency_paths, dependency)

        if dependency_path:
            return dependency_path

        raise Exception(
            f"Linux Dependency {dependency.dependency_name}=={dependency.dependency_version} not found in {directory} directory"
        )

    def get_dependency_filename_from_path(self, dependency_path: Path):
        return dependency_path.name

    def __get_dependency_name_from_path(self, dependency_path: Path):
        return dependency_path.stem.split("-")[0].replace("_", "-")

    def __get_dependency_version_from_path(self, dependency_path: Path):
        return dependency_path.stem.split("-")[1]

    def __get_dependency_platform_from_path(self, dependency_path: Path):
        return dependency_path.stem.split("-")[-1]

    def __get_all_dependency_paths_from_dir(self, directory: Path):
        return directory.glob("*.whl")

    def __get_linux_dependency_paths_from_dir(self, directory: Path):
        return [dependency_path for dependency_path in directory.glob("*.whl") if self.__is_linux_dependency(dependency_path)]

    def __is_linux_dependency(self, dependency_path: Path):
        platform = self.__get_dependency_platform_from_path(dependency_path).lower()

        return platform == "any" or "linux" in platform

    def __find_dependency_path(self, dependency_paths: Iterable[Path], dependency: Dependency):
        for dependency_path in dependency_paths:
            wheel_name = self.__get_dependency_name_from_path(dependency_path)
            wheel_version = self.__get_dependency_version_from_path(dependency_path)

            if wheel_name.lower() == dependency.dependency_name.lower() and wheel_version == dependency.dependency_version:
                return dependency_path

        return None



from requests.exceptions import HTTPError
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from pathlib import PurePosixPath
import base64
from dbxdeploy.workspace.WorkspaceExportException import WorkspaceExportException


class WorkspaceExporter:
    def __init__(self, dbx_api: DatabricksClient):
        self.__dbx_api = dbx_api

    def export(self, path: PurePosixPath) -> bytes:
        try:
            response = self.__dbx_api.workspace.export_workspace(str(path), format="DBC")
        except HTTPError:
            raise WorkspaceExportException()

        return base64.decodebytes(response["content"].encode("utf-8"))



import os
from pathlib import Path


def delete_directory_content_recursive(dir_path: Path):
    if dir_path.is_dir():
        file_list = []
        dir_list = []

        for root, dirs, files in os.walk(dir_path):
            for f in files:
                file_list.append(os.path.join(root, f))

            for d in dirs:
                dir_list.append(os.path.join(root, d))

        for f in file_list:
            os.unlink(f)

        for d in dir_list:
            os.rmdir(d)



from argparse import Namespace
from logging import Logger
from pathlib import PurePosixPath, Path
from zipfile import ZipInfo, ZipFile
from consolebundle.ConsoleCommand import ConsoleCommand
from dbxdeploy.dbc.DbcNotebookConverter import DbcNotebookConverter
from dbxdeploy.notebook.converter.NotebookConverterInterface import NotebookConverterInterface
from dbxdeploy.notebook.converter.UnexpectedSourceException import UnexpectedSourceException
from dbxdeploy.notebook.loader import load_notebook
from dbxdeploy.workspace.DbcFilesHandler import DbcFilesHandler
from dbxdeploy.workspace.WorkspaceExporter import WorkspaceExporter
from dbxdeploy.black.BlackChecker import BlackChecker


class WorkspaceExportCommand(ConsoleCommand):
    def __init__(
        self,
        workspace_base_dir: PurePosixPath,
        project_base_dir: Path,
        relative_base_dir: str,
        logger: Logger,
        notebook_converter: NotebookConverterInterface,
        workspace_exporter: WorkspaceExporter,
        dbc_files_handler: DbcFilesHandler,
        dbc_notebook_converter: DbcNotebookConverter,
        black_checker: BlackChecker,
    ):
        self.__workspace_base_dir = workspace_base_dir
        self.__local_base_dir = project_base_dir.joinpath(relative_base_dir)
        self.__logger = logger
        self.__notebook_converter = notebook_converter
        self.__workspace_exporter = workspace_exporter
        self.__dbc_files_handler = dbc_files_handler
        self.__dbc_notebook_converter = dbc_notebook_converter
        self.__black_checker = black_checker

    def get_command(self) -> str:
        return "dbx:workspace:export"

    def get_description(self):
        return "Export notebooks from Databricks workspace to local project"

    def run(self, input_args: Namespace):
        self.__logger.info(f"Exporting {self.__workspace_base_dir} to {self.__local_base_dir}")

        if self.__black_checker.is_black_enabled and not self.__black_checker.is_black_installed:
            self.__logger.warning("Black enabled, but not installed. Skipping black formatting.")

        dbc_content = self.__workspace_exporter.export(self.__workspace_base_dir)
        self.__dbc_files_handler.handle(dbc_content, self.__read_file)

        self.__logger.info("Export completed")

    def __read_file(self, zip_file: ZipFile, file: ZipInfo):
        if file.orig_filename[-1:] == "/":
            return

        file_path_without_rootdir = (
            file.orig_filename[file.orig_filename.index("/") + 1 : file.orig_filename.rindex(".")] + ".py"  # noqa: 5203
        )
        local_file_path = self.__local_base_dir.joinpath(file_path_without_rootdir)

        if local_file_path.exists():
            local_file_source = load_notebook(local_file_path)

            try:
                self.__notebook_converter.validate_source(local_file_source)
            except UnexpectedSourceException:
                self.__logger.error(f"Skipping unrecognized file {local_file_path}")
                return

        if self.__black_checker.is_black_enabled and self.__black_checker.is_black_installed:
            try:
                self.__dbc_notebook_converter.convert(zip_file, file)

            except Exception as ex:
                self.__logger.error(f"Black cannot format file {file.orig_filename}, skipping...")
                self.__logger.error(ex)
                return

        py_content = self.__dbc_notebook_converter.convert(zip_file, file)
        self.__write_py_content(local_file_path, py_content)

    def __write_py_content(self, local_file_path, py_content):
        if not local_file_path.parent.exists():
            local_file_path.parent.mkdir(parents=True)

        with local_file_path.open("wb") as f:
            f.write(py_content)



from dbxdeploy.deploy.CurrentAndReleaseDeployer import CurrentAndReleaseDeployer
from dbxdeploy.package.PackageMetadataLoader import PackageMetadataLoader
from dbxdeploy.package.PackageDeployer import PackageDeployer
from pathlib import Path
import asyncio


class Deployer:
    def __init__(
        self,
        project_base_dir: Path,
        package_metadata_loader: PackageMetadataLoader,
        current_and_release_deployer: CurrentAndReleaseDeployer,
        package_deployer: PackageDeployer,
    ):
        self.__project_base_dir = project_base_dir
        self.__package_metadata_loader = package_metadata_loader
        self.__current_and_release_deployer = current_and_release_deployer
        self.__package_deployer = package_deployer

    async def deploy(self):
        package_metadata = self.__package_metadata_loader.load(self.__project_base_dir)

        loop = asyncio.get_event_loop()

        package_deploy_future = loop.run_in_executor(None, self.__package_deployer.deploy, package_metadata)
        dbc_deploy_future = loop.run_in_executor(None, self.__current_and_release_deployer.deploy, package_metadata)

        await package_deploy_future
        await dbc_deploy_future



from logging import Logger
from pathlib import PurePosixPath, Path
from typing import List
import zipfile
from io import BytesIO
from dbxdeploy.dbc.PathsPreparer import PathsPreparer
from dbxdeploy.notebook.Notebook import Notebook
from dbxdeploy.notebook.converter.NotebookConverterInterface import NotebookConverterInterface
from dbxdeploy.notebook.converter.UnexpectedSourceException import UnexpectedSourceException
from dbxdeploy.notebook.loader import load_notebook


class DbcCreator:
    def __init__(
        self,
        working_directory: Path,
        logger: Logger,
        notebook_converter: NotebookConverterInterface,
        paths_preparer: PathsPreparer,
    ):
        self.__working_directory = working_directory
        self.__logger = logger
        self.__notebook_converter = notebook_converter
        self.__paths_preparer = paths_preparer

    def create(self, notebooks: List[Notebook], package_file_path: str, dependencies_dir_path: str) -> bytes:
        databricks_relative_paths = list(map(lambda notebook: notebook.databricks_relative_path, notebooks))
        root_ignored_path_name = "root_ignored_path"

        in_memory_output = BytesIO()

        zip_file = zipfile.ZipFile(in_memory_output, "w", zipfile.ZIP_DEFLATED)

        # directories must be created first, otherwise DataBricks is not able to process that zip/dbc file
        for dir_path in self.__paths_preparer.prepare(databricks_relative_paths, root_ignored_path_name):
            zip_file.writestr(dir_path + "/", "")

        for notebook in notebooks:
            source = load_notebook(notebook.path)

            try:
                self.__notebook_converter.validate_source(source)
            except UnexpectedSourceException:
                self.__logger.debug(f"Skipping unrecognized file {notebook.relative_path}")
                continue

            notebook_source = self.__notebook_converter.to_dbc_notebook(
                notebook.path.stem, source, package_file_path, dependencies_dir_path
            )
            zip_path = PurePosixPath(root_ignored_path_name).joinpath(notebook.databricks_relative_path).with_suffix(".python")
            zip_file.writestr(str(zip_path), notebook_source)

        zip_file.close()
        in_memory_output.seek(0)

        zip_content = in_memory_output.getvalue()
        dist_dir = self.__working_directory.joinpath("dist")

        if not dist_dir.exists():
            dist_dir.mkdir()

        with dist_dir.joinpath("notebooks.dbc").open("wb") as f:
            f.write(zip_content)

        return zip_content



from dbxdeploy.utils.DatabricksClient import DatabricksClient
from logging import Logger


class JobsDeleter:
    def __init__(self, logger: Logger, dbx_api: DatabricksClient):
        self.__logger = logger
        self.__dbx_api = dbx_api

    def remove_all(self):
        def callback(job_id, notebook_path):
            self.__logger.info("Deleting job #{}: {}".format(job_id, notebook_path))
            self.__dbx_api.jobs.delete_job(job_id)

        self.__remove(callback)

    def remove(self, notebook_paths: set):
        def callback(job_id, notebook_path):
            if notebook_path in notebook_paths:
                self.__logger.info("Deleting job #{}: {}".format(job_id, notebook_path))
                self.__dbx_api.jobs.delete_job(job_id)

        self.__remove(callback)

    def __remove(self, callback: callable):
        while True:
            jobs_response = self.__dbx_api.jobs.list_jobs()

            if "jobs" not in jobs_response:
                if "jobs" in locals():
                    self.__logger.info("No more jobs exist")
                else:
                    self.__logger.info("No jobs exist")
                break

            jobs = jobs_response["jobs"]

            self.__logger.info("{} active jobs found".format(len(jobs)))

            for job in jobs:
                job_id = job["job_id"]
                notebook_path = job["settings"]["notebook_task"]["notebook_path"]

                callback(job_id, notebook_path)



from abc import ABC, abstractmethod


class PackageUploaderInterface(ABC):
    @abstractmethod
    def upload(self, content: bytes, file_path: str, overwrite: bool = False):
        pass

    @abstractmethod
    def exists(self, file_path: str):
        pass



import re
from dbxdeploy.dbc.CommandsConverter import CommandsConverter
from dbxdeploy.notebook.converter import empty_lines_remover
from dbxdeploy.notebook.converter.CellsExtractor import CellsExtractor
from dbxdeploy.notebook.converter.DbcScriptRenderer import DbcScriptRenderer
from dbxdeploy.notebook.converter.JinjaTemplateLoader import JinjaTemplateLoader
from dbxdeploy.notebook.converter.UnexpectedSourceException import UnexpectedSourceException
from dbxdeploy.package.CodeStyleLoader import CodeStyleLoader
from dbxdeploy.package.PackageInstaller import PackageInstaller
from dbxdeploy.notebook.converter.NotebookConverterInterface import NotebookConverterInterface


class CommandSeparatorConverter(NotebookConverterInterface):

    first_line = "# Databricks notebook source"
    cell_separator = "# COMMAND ----------"

    def __init__(
        self,
        commands_converter: CommandsConverter,
        cells_extractor: CellsExtractor,
        jinja_template_loader: JinjaTemplateLoader,
        dbc_script_renderer: DbcScriptRenderer,
        package_installer: PackageInstaller,
        codestyle_loader: CodeStyleLoader,
    ):
        self.__commands_converter = commands_converter
        self.__cells_extractor = cells_extractor
        self.__jinja_template_loader = jinja_template_loader
        self.__dbc_script_renderer = dbc_script_renderer
        self.__package_installer = package_installer
        self.__codestyle_loader = codestyle_loader

    def validate_source(self, source: str):
        if re.match(r"^" + self.first_line + "[\r\n]", source) is None:
            raise UnexpectedSourceException()

    def from_dbc_notebook(self, content: dict) -> str:
        return self.__commands_converter.convert(content["commands"], self.first_line, self.cell_separator)

    def to_dbc_notebook(self, notebook_name: str, source: str, package_file_path: str, dependencies_dir_path: str) -> str:
        cells = self.__cells_extractor.extract(source, r"#[\s]+COMMAND[\s]+[\-]+\n+")

        def cleanup_cell(cell: dict):
            if cell["source"] == "# MAGIC %install_master_package_whl":
                cell["source"] = self.__package_installer.get_package_install_command(package_file_path, dependencies_dir_path)

            if cell["source"] == f"# MAGIC {self.__codestyle_loader.get_setup_command()}":
                cell["source"] = (
                    f"{self.__codestyle_loader.get_codestyle_install_command()}\n"
                    f"{self.cell_separator}\n"
                    f"{self.__codestyle_loader.get_codestyle_setup_command()}"
                )

            cell["source"] = re.sub(r"^" + self.first_line + "[\r\n]+", "", cell["source"])
            cell["source"] = re.sub(r"# MAGIC\s*", "", cell["source"])
            cell["source"] = empty_lines_remover.remove(cell["source"])

            return cell

        cells = list(map(cleanup_cell, cells))

        template = self.__jinja_template_loader.load()

        return self.__dbc_script_renderer.render(notebook_name, template, cells)

    def to_workspace_import_notebook(self, source: str, package_file_path: str, dependencies_dir_path: str) -> str:
        source = empty_lines_remover.remove(source)
        source = source.replace(
            "# MAGIC %install_master_package_whl",
            self.__package_installer.get_package_install_command(package_file_path, dependencies_dir_path),
        )
        if self.__codestyle_loader.get_setup_command() in source:
            source = source.replace(
                f"# MAGIC {self.__codestyle_loader.get_setup_command()}",
                f"{self.__codestyle_loader.get_codestyle_install_command()}\n"
                f"{self.cell_separator}\n"
                f"{self.__codestyle_loader.get_codestyle_setup_command()}",
            )

        return source



import re


def remove(source: str):
    source = re.sub(r"(#[\s]+COMMAND[\s]+[-]+)([\n]|[\r\n]|[\r]){3,}", "\\1\\2\\2", source, flags=re.MULTILINE)
    source = re.sub(r"([\n]|[\r\n]|[\r]){3,}(#[\s]+COMMAND[\s]+[-]+)", "\\1\\1\\2", source, flags=re.MULTILINE)

    return source



from logging import Logger
from urllib3 import get_host
from databricks_cli.repos.api import ReposApi
from databricks_cli.workspace.api import WorkspaceApi


class RepoManager:
    def __init__(
        self,
        repo_root_dir: str,
        repo_path: str,
        logger: Logger,
        repos_api: ReposApi,
        workspace_api: WorkspaceApi,
    ):
        self.__repos_api = repos_api
        self.__workspace_api = workspace_api
        self.__logger = logger
        self.__repo_root_dir = repo_root_dir
        self.__repo_path = repo_path

    def create_or_update(self, repo_url, checkout_branch, repo_name):
        repo_path = self.__repo_path.format(repo_name=repo_name)

        self.__logger.info(f"Trying to update repo at {repo_path}")
        self.__workspace_api.mkdirs(self.__repo_root_dir)

        try:
            repo_id = self.__repos_api.get_repo_id(repo_path)
            self.__check_for_duplicates()
            self.__check_correct_project(repo_id, repo_url)
        except RuntimeError:
            self.__logger.info("Repo not found, cloning new repo")
            self.__check_for_duplicates(with_branch=checkout_branch)
            repo_id = self.__repos_api.create(url=repo_url, provider=self.__get_provider_from_url(repo_url), path=repo_path)["id"]

        self.__repos_api.update(repo_id=repo_id, branch=checkout_branch, tag=None)
        self.__logger.info("Repo successfully updated")

    def delete(self, repo_url, repo_name):
        repo_path = self.__repo_path.format(repo_name=repo_name)

        self.__logger.info(f"Trying to delete repo at {repo_path}")

        try:
            repo_id = self.__repos_api.get_repo_id(repo_path)
            self.__check_correct_project(repo_id, repo_url)
        except RuntimeError:
            self.__logger.info("Repo not found, skipping delete")
        else:
            self.__repos_api.delete(repo_id)
            self.__logger.info("Repo successfully deleted")

        self.__delete_root_dir_if_empty()

    def __check_correct_project(self, repo_id, repo_url):
        repo = self.__repos_api.get(repo_id)
        if repo_url != repo["url"]:
            raise Exception(f"Repo at {repo['path']} has source at {repo['url']}, expected {repo_url}")

    def __check_for_duplicates(self, with_branch: str = None):
        repo_list = self.__repos_api.list(self.__repo_root_dir, None).get("repos", [])
        repo_branches = [repo["branch"] for repo in repo_list]
        if with_branch:
            repo_branches.append(with_branch)
        branch_counter = [{"name": branch, "count": repo_branches.count(branch)} for branch in set(repo_branches)]
        duplicated_branches = [branch["name"] for branch in branch_counter if branch["count"] > 1]

        if len(duplicated_branches) > 0:
            raise Exception(f"Following branches are/would be duplicated in current env: {', '.join(duplicated_branches)}.")

    def __delete_root_dir_if_empty(self):
        repos = self.__workspace_api.list_objects(self.__repo_root_dir)
        if len(repos) < 1:
            self.__workspace_api.delete(self.__repo_root_dir, is_recursive=False)

    def __get_provider_from_url(self, url):
        provider_map = {
            "github.com": "gitHub",
            "dev.azure.com": "azureDevOpsServices",
        }
        try:
            return provider_map[get_host(url)[1]]
        except KeyError:
            raise Exception(f"Git provider for {url} not listed.")



import sys
import shutil
from pathlib import Path
from dbxdeploy.package.BootstrapConfigAppender import BootstrapConfigAppender
from dbxdeploy.package.LockedPyprojectCreator import LockedPyprojectCreator
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.deploy.LocalPathsResolver import LocalPathsResolver
from dbxdeploy.poetry.PoetryPathResolver import PoetryPathResolver
from dbxdeploy.shell.runner import run_shell_command
from dbxdeploy.filesystem.utils import delete_directory_content_recursive


class PackageBuilder:
    def __init__(
        self,
        project_base_dir: Path,
        offline_install: bool,
        locked_pyproject_creator: LockedPyprojectCreator,
        bootstrap_config_appender: BootstrapConfigAppender,
        local_paths_resolver: LocalPathsResolver,
        poetry_path_resolver: PoetryPathResolver,
    ):
        self.__project_base_dir = project_base_dir
        self.__offline_install = offline_install
        self.__locked_pyproject_creator = locked_pyproject_creator
        self.__bootstrap_config_appender = bootstrap_config_appender
        self.__local_paths_resolver = local_paths_resolver
        self.__poetry_path_resolver = poetry_path_resolver

    def build(self, base_path: Path, package_metadata: PackageMetadata):
        dist_dir = base_path.joinpath("dist")

        delete_directory_content_recursive(dist_dir)

        pyproject_orig_path = base_path.joinpath("pyproject.toml")
        pyproject_new_path = base_path.joinpath("pyproject.toml.new")

        self.__locked_pyproject_creator.create(package_metadata, pyproject_orig_path, pyproject_new_path)

        package_file_name = package_metadata.get_package_filename()
        package_path = self.__project_base_dir.joinpath(Path("dist")).joinpath(package_file_name)

        self.__build_wheel(pyproject_orig_path, pyproject_new_path, base_path)

        self.__bootstrap_config_appender.append(package_path)

        if self.__offline_install:
            self.__build_wheelhouse(base_path, package_metadata)

    def __build_wheel(self, pyproject_orig_path: Path, pyproject_new_path: Path, base_path: Path):
        pyproject_dist = base_path.joinpath("dist/pyproject.toml")
        pyproject_bak_path = base_path.joinpath("pyproject.toml.bak")

        pyproject_orig_path.rename(pyproject_bak_path)
        pyproject_new_path.rename(pyproject_orig_path)

        python_executable = sys.executable
        poetry_path = self.__poetry_path_resolver.get_poetry_path()

        try:
            run_shell_command(f"{python_executable} {poetry_path} build --format wheel", cwd=str(base_path), shell=True)
        except BaseException:
            pass
        finally:
            pyproject_orig_path.replace(pyproject_dist)
            pyproject_bak_path.rename(pyproject_orig_path)

    def __build_wheelhouse(self, base_path: Path, package_metadata: PackageMetadata):
        dependencies_dir = base_path.joinpath("dependencies")
        dist_dir = base_path.joinpath("dist")

        if not dependencies_dir.is_dir():
            raise Exception(f"Cannot find dependencies dir at {dependencies_dir}")

        for dependency in package_metadata.dependencies:
            dependency_wheel = self.__local_paths_resolver.get_linux_dependency_path_from_dir(dependencies_dir, dependency)

            shutil.copy(dependency_wheel, dist_dir)



import json
import zipfile
from pyfonycore import pyproject
from pyfonycore.bootstrap.config.raw import raw_config_reader
from pathlib import Path


class BootstrapConfigAppender:
    def append(self, package_path: Path):
        package_file_name = package_path.stem
        package_file_name_parts = package_file_name.split("-")
        package_name = package_file_name_parts[0].replace("__", "_")  # __myproject__ needs to be converted to _myproject_ for some reason

        dist_info_dir = f"{package_name}-{package_file_name_parts[1]}.dist-info"

        zip_file = zipfile.ZipFile(package_path, "a")
        zip_file.writestr(dist_info_dir + "/entry_points.txt", self.__prepare_entry_points(package_name))
        zip_file.writestr(dist_info_dir + "/bootstrap_config.json", self.__prepare_bootstrap_config())
        zip_file.close()

    def __prepare_entry_points(self, package_name):
        return f"[pyfony.bootstrap]\npackage_name={package_name}\n"

    def __prepare_bootstrap_config(self):
        bootstrap_config = raw_config_reader.read(pyproject.get_path())
        return json.dumps(bootstrap_config)



from typing import List
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from dbxdeploy.job.JobCreator import JobCreator
from dbxdeploy.notebook.Notebook import Notebook
from dbxdeploy.package.PackageMetadata import PackageMetadata
from logging import Logger


class JobsCreatorAndRunner:
    def __init__(
        self,
        logger: Logger,
        dbx_api: DatabricksClient,
        job_creator: JobCreator,
    ):
        self.__logger = logger
        self.__dbx_api = dbx_api
        self.__job_creator = job_creator

    def create_and_run(self, notebooks: List[Notebook], cluster_id: str, package_metadata: PackageMetadata):
        created_jobs = list(
            map(lambda notebook: self.__job_creator.create(notebook.databricks_relative_path, cluster_id, package_metadata), notebooks)
        )

        self.__logger.info("--")

        for created_job in created_jobs:
            job_id = created_job["job_id"]
            notebook_release_path = created_job["notebook_release_path"]

            self.__logger.info("Running job #{}: {}".format(job_id, notebook_release_path))
            self.__dbx_api.jobs.run_now(job_id)

        self.__logger.info("--")



from dbxnotebookexporter.json.formatCellContent import formatCellContent
import dbxnotebookexporter.json.JsonNotebookExporter as JsonNotebookExporterModule
import nbconvert.templates.skeleton as nb_convert_skeleton
import jinja2
import os


class JinjaTemplateLoader:
    def load(self):
        base_paths = [
            os.path.dirname(JsonNotebookExporterModule.__file__),
            nb_convert_skeleton.__path__._path[0],
        ]

        template_loader = jinja2.FileSystemLoader(searchpath=base_paths)
        template_env = jinja2.Environment(loader=template_loader)
        template_env.filters["formatCellContent"] = formatCellContent

        return template_env.get_template("json_notebook.tpl")



from base64 import b64encode
from pathlib import PurePosixPath
from dbxdeploy.utils.DatabricksClient import DatabricksClient
import os


class WorkspaceImporter:
    def __init__(
        self,
        dbx_api: DatabricksClient,
    ):
        self.__dbx_api = dbx_api

    def overwrite_script(self, script: str, target_path: PurePosixPath):
        content_to_upload = b64encode(script.encode("utf-8"))

        self.__dbx_api.workspace.mkdirs(os.path.dirname(str(target_path)))

        self.__dbx_api.workspace.import_workspace(
            str(target_path), format="SOURCE", language="PYTHON", content=str(content_to_upload, "utf-8"), overwrite=True
        )



from typing import List
from jinja2 import Template


class DbcScriptRenderer:
    def render(self, notebook_name: str, template: Template, cells: List[dict]):
        resources = {
            "metadata": {
                "name": notebook_name,
            },
            "global_content_filter": {
                "include_code": True,
                "include_input": False,
                "include_input_prompt": False,
                "include_output": False,
            },
        }

        return template.render(resources=resources, nb={"cells": cells})



class BlackChecker:
    def __init__(self, black_enabled: bool):
        self.__black_enabled = black_enabled

    @property
    def is_black_enabled(self) -> bool:
        return self.__black_enabled

    @property
    def is_black_installed(self) -> bool:
        try:
            import black  # noqa

            return True

        except ImportError:
            return False



import tomlkit
from tomlkit.toml_document import TOMLDocument
from tomlkit.items import Table
from typing import List
from pathlib import Path


class PackageIndexResolver:
    def __init__(
        self,
        project_base_dir: Path,
    ):
        self.__project_base_dir = project_base_dir

    def get_default_index(self) -> Table:
        toml_doc = self.__load_pyproject_toml()

        if "source" not in toml_doc["tool"]["poetry"]:
            return None

        sources = toml_doc["tool"]["poetry"]["source"]

        for source in sources:
            if source.get("default") is True:
                return source

        return None

    def get_secondary_indexes(self) -> List[Table]:
        toml_doc = self.__load_pyproject_toml()

        if "source" not in toml_doc["tool"]["poetry"]:
            return None

        sources = toml_doc["tool"]["poetry"]["source"]
        extra_indexes = []

        for source in sources:
            if source.get("secondary") is True:
                extra_indexes.append(source)

        if len(extra_indexes) > 0:
            return extra_indexes

        return None

    def has_default_index(self) -> bool:
        return self.get_default_index() is not None

    def has_secondary_indexes(self) -> bool:
        return self.get_secondary_indexes() is not None

    def __load_pyproject_toml(self) -> TOMLDocument:
        pyproject_path = self.__project_base_dir.joinpath("pyproject.toml")

        with pyproject_path.open("r") as f:
            toml_doc = tomlkit.parse(f.read())

        return toml_doc



from urllib.parse import urlparse
from dbxdeploy.package.PackageIndexResolver import PackageIndexResolver


class PackageInstaller:
    def __init__(
        self,
        package_base_dir: str,
        offline_install: bool,
        package_index_resolver: PackageIndexResolver,
    ):
        self.__package_base_dir = package_base_dir
        self.__offline_install = offline_install
        self.__package_index_resolver = package_index_resolver

    def get_package_install_command(self, package_file_path: str, dependencies_dir_path: str):
        if self.__offline_install:
            return self.__get_offline_install_command(package_file_path, dependencies_dir_path)

        return self.__get_online_install_command(package_file_path)

    def is_package_install_command(self, command_code: str):
        return command_code.startswith("# %install_master_package_whl")

    def __modify_dbfs(self, path: str):
        return "/dbfs/" + path.lstrip("dbfs:/")

    def __get_install_command(self, package_file_path: str, options: [str]):
        pip_options = " ".join(options)

        return (
            "# %install_master_package_whl\n"
            "import IPython\n"
            "import os\n"
            "ipy = IPython.get_ipython()\n"
            "ipy.run_line_magic"
            f"('pip', 'install {self.__modify_dbfs(package_file_path)} {pip_options}') # noqa E501"
        )

    def __get_online_install_command(self, package_file_path: str):
        options = []

        if self.__package_index_resolver.has_default_index():
            options.append(f"{self.__get_index_url_part()}")

        if self.__package_index_resolver.has_secondary_indexes():
            options.append(f"{self.__get_extra_index_url_part()}")

        install_command = self.__get_install_command(package_file_path, options)

        return install_command

    def __get_offline_install_command(self, package_file_path: str, dependencies_dir_path: str):
        options = ["--no-index", f"--find-links {self.__modify_dbfs(dependencies_dir_path)}"]
        install_command = self.__get_install_command(package_file_path, options)

        return install_command

    def __get_index_url_part(self):
        default_index = self.__package_index_resolver.get_default_index()
        default_index_url = default_index["url"]
        username_env_var_name = f'DATABRICKS_HTTP_BASIC_{default_index["name"]}_USERNAME'.upper()
        password_env_var_name = f'DATABRICKS_HTTP_BASIC_{default_index["name"]}_PASSWORD'.upper()

        parsed_url = urlparse(default_index_url)

        if "@" not in parsed_url.netloc:
            parsed_url = parsed_url._replace(
                netloc=f'{{os.getenv("{username_env_var_name}")}}:{{os.getenv("{password_env_var_name}")}}@{parsed_url.netloc}'
            )

        return f"--index-url {parsed_url.geturl()}"

    def __get_extra_index_url_part(self):
        extra_indexes = self.__package_index_resolver.get_secondary_indexes()
        extra_indexes_to_return = []

        for extra_index in extra_indexes:
            extra_index_url = extra_index["url"]
            username_env_var_name = f'DATABRICKS_HTTP_BASIC_{extra_index["name"]}_USERNAME'.upper()
            password_env_var_name = f'DATABRICKS_HTTP_BASIC_{extra_index["name"]}_PASSWORD'.upper()

            parsed_url = urlparse(extra_index_url)

            if "@" not in parsed_url.netloc:
                parsed_url = parsed_url._replace(
                    netloc=f'{{os.getenv("{username_env_var_name}")}}:{{os.getenv("{password_env_var_name}")}}@{parsed_url.netloc}'
                )

            extra_indexes_to_return.append(f"--extra-index-url {parsed_url.geturl()}")

        return " ".join(extra_indexes_to_return)



from logging import Logger
from argparse import Namespace, ArgumentParser
from typing import Optional
from dbxdeploy.job.JobsDeleter import JobsDeleter
from dbxdeploy.cluster.ClusterRestarter import ClusterRestarter
from consolebundle.ConsoleCommand import ConsoleCommand


class JobsDeleterCommand(ConsoleCommand):
    def __init__(
        self,
        cluster_id: Optional[str],
        logger: Logger,
        cluster_restarter: ClusterRestarter,
        jobs_deleter: JobsDeleter,
    ):
        self.__cluster_id = cluster_id
        self.__logger = logger
        self.__cluster_restarter = cluster_restarter
        self.__jobs_deleter = jobs_deleter

    def get_command(self) -> str:
        return "dbx:jobs:delete-all"

    def get_description(self):
        return "Deletes all DBX jobs"

    def configure(self, argument_parser: ArgumentParser):
        # deprecated, will be removed in 2.0; use CLI argument to defined cluster_id instead
        if self.__cluster_id is None:
            argument_parser.add_argument(dest="cluster_id", help="Cluster to be restarted before job deletion starts")

    def run(self, input_args: Namespace):
        self.__logger.info("Jobs deletion started")

        cluster_id = self.__cluster_id if self.__cluster_id is not None else input_args.cluster_id

        self.__cluster_restarter.restart(cluster_id)
        self.__jobs_deleter.remove_all()



from pathlib import PurePosixPath
from dbxdeploy.job.RunsGetter import RunsGetter
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from logging import Logger


class NotebookKiller:
    def __init__(
        self,
        logger: Logger,
        dbx_api: DatabricksClient,
        runs_getter: RunsGetter,
    ):
        self.__logger = logger
        self.__dbx_api = dbx_api
        self.__runs_getter = runs_getter

    def kill_if_running(self, notebook_path: PurePosixPath, cluster_id: str, package_metadata: PackageMetadata):
        self.__logger.info("Looking for jobs running the {} notebook".format(notebook_path))
        previous_notebook_runs = self.__runs_getter.get(notebook_path, cluster_id, package_metadata)

        if previous_notebook_runs:
            for running_job in previous_notebook_runs:
                self.__logger.warning(
                    "Killing JOB #{} for {}".format(str(running_job["run_id"]), running_job["task"]["notebook_task"]["notebook_path"])
                )
                self.__dbx_api.jobs.cancel_run(running_job["run_id"])
        else:
            self.__logger.info("notebook {} is not running in any job".format(notebook_path))



from dbxdeploy.git.CurrentRepositoryFactory import CurrentRepositoryFactory


class CurrentBranchResolver:
    def __init__(
        self,
        current_repository_factory: CurrentRepositoryFactory,
    ):
        self.__current_repository_factory = current_repository_factory

    def resolve(self) -> str:
        return self.__current_repository_factory.create().head.shorthand



from dbxdeploy.package.PackageBuilder import PackageBuilder
from dbxdeploy.package.PackageMetadata import PackageMetadata
from pathlib import Path
from logging import Logger
from dbxdeploy.deploy.TargetPathsResolver import TargetPathsResolver
from dbxdeploy.deploy.LocalPathsResolver import LocalPathsResolver
from dbxdeploy.package.PackageUploaderInterface import PackageUploaderInterface


class PackageDeployer:
    def __init__(
        self,
        project_base_dir: Path,
        offline_install: bool,
        logger: Logger,
        package_uploader: PackageUploaderInterface,
        package_builder: PackageBuilder,
        target_paths_resolver: TargetPathsResolver,
        local_paths_resolver: LocalPathsResolver,
    ):
        self.__project_base_dir = project_base_dir
        self.__offline_install = offline_install
        self.__logger = logger
        self.__package_uploader = package_uploader
        self.__package_builder = package_builder
        self.__target_paths_resolver = target_paths_resolver
        self.__local_paths_resolver = local_paths_resolver

    def deploy(self, package_metadata: PackageMetadata):
        def whl_content_ready_callback():
            if self.__offline_install:
                for dependency in package_metadata.dependencies:
                    dependency_local_path = self.__local_paths_resolver.get_dependency_dist_path(dependency)
                    dependency_filename = self.__local_paths_resolver.get_dependency_filename_from_path(dependency_local_path)
                    dependency_deploy_path = self.__target_paths_resolver.get_dependency_upload_path_for_deploy(
                        package_metadata, dependency_filename
                    )

                    if not self.__package_uploader.exists(dependency_deploy_path):
                        self.__upload(dependency_local_path, dependency_deploy_path)
                    else:
                        self.__logger.debug(f"Package at {dependency_deploy_path} already exists. Skipping...")

            master_package_local_path = self.__local_paths_resolver.get_package_dist_path(package_metadata)
            master_package_deploy_path = self.__target_paths_resolver.get_package_upload_path_for_deploy(package_metadata)

            self.__upload(master_package_local_path, master_package_deploy_path)

        self.__invoke(package_metadata, whl_content_ready_callback)

    def release(self, package_metadata: PackageMetadata):
        def whl_content_ready_callback():
            if self.__offline_install:
                for dependency in package_metadata.dependencies:
                    dependency_local_path = self.__local_paths_resolver.get_dependency_dist_path(dependency)
                    dependency_filename = self.__local_paths_resolver.get_dependency_filename_from_path(dependency_local_path)
                    dependency_release_path = self.__target_paths_resolver.get_dependency_upload_path_for_release(
                        package_metadata, dependency_filename
                    )

                    self.__upload(dependency_local_path, dependency_release_path)

            master_package_local_path = self.__local_paths_resolver.get_package_dist_path(package_metadata)
            master_package_release_path = self.__target_paths_resolver.get_package_upload_path_for_release(package_metadata)

            self.__upload(master_package_local_path, master_package_release_path)

        self.__invoke(package_metadata, whl_content_ready_callback)

    def __invoke(self, package_metadata: PackageMetadata, whl_content_ready_callback: callable):
        self.__logger.info("Building master package (WHL)")

        self.__package_builder.build(self.__project_base_dir, package_metadata)

        whl_content_ready_callback()

        self.__logger.info("App package uploaded")

    def __upload(self, local_path: Path, target_path: str):
        self.__logger.info(f"Uploading WHL package to {target_path}")

        with local_path.open("rb") as file:
            self.__package_uploader.upload(file.read(), target_path, overwrite=True)



from pathlib import PurePosixPath
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from logging import Logger
from dbxdeploy.deploy.TargetPathsResolver import TargetPathsResolver


class JobCreator:
    def __init__(
        self,
        logger: Logger,
        dbx_api: DatabricksClient,
        target_paths_resolver: TargetPathsResolver,
    ):
        self.__logger = logger
        self.__dbx_api = dbx_api
        self.__target_paths_resolver = target_paths_resolver

    def create(self, notebook_path: PurePosixPath, cluster_id: str, package_metadata: PackageMetadata):
        notebook_release_path = self.__target_paths_resolver.get_workspace_release_path(package_metadata) / notebook_path

        self.__logger.info("Creating job for {}".format(str(notebook_release_path)))

        job = self.__dbx_api.jobs.create_job(
            name=str(notebook_path),
            existing_cluster_id=cluster_id,
            max_retries=-1,
            min_retry_interval_millis=30 * 1000,
            retry_on_timeout=True,
            notebook_task=dict(notebook_path=str(notebook_release_path)),
            max_concurrent_runs=1,
        )

        self.__logger.info("job #{} created".format(job["job_id"]))

        job["notebook_release_path"] = notebook_release_path

        return job



from pathlib import Path
from argparse import Namespace
from dbxdeploy.package.PackageMetadataLoader import PackageMetadataLoader
from dbxdeploy.package.PackageDeployer import PackageDeployer
from dbxdeploy.deploy.CurrentAndReleaseDeployer import CurrentAndReleaseDeployer
from consolebundle.ConsoleCommand import ConsoleCommand


class MasterPackageDeployer(ConsoleCommand):
    def __init__(
        self,
        project_base_dir: Path,
        package_metadata_loader: PackageMetadataLoader,
        package_deployer: PackageDeployer,
        current_and_release_deployer: CurrentAndReleaseDeployer,
    ):
        self.__project_base_dir = project_base_dir
        self.__package_metadata_loader = package_metadata_loader
        self.__package_deployer = package_deployer
        self.__current_and_release_deployer = current_and_release_deployer

    def get_command(self) -> str:
        return "dbx:deploy-master-package"

    def get_description(self):
        return "Deploy WHL to DBX"

    def run(self, input_args: Namespace):
        package_metadata = self.__package_metadata_loader.load(self.__project_base_dir)

        self.__package_deployer.deploy(package_metadata)
        self.__current_and_release_deployer.deploy_only_master_package_notebook(package_metadata)



import json
from zipfile import ZipInfo, ZipFile
from dbxdeploy.notebook.converter.NotebookConverterInterface import NotebookConverterInterface


class DbcNotebookConverter:
    def __init__(
        self,
        notebook_converter: NotebookConverterInterface,
    ):
        self.__notebook_converter = notebook_converter

    def convert(self, zip_file: ZipFile, file: ZipInfo):
        zipped_file_content = zip_file.read(file.orig_filename).decode("utf-8")
        py_content = self.__notebook_converter.from_dbc_notebook(json.loads(zipped_file_content))

        return py_content.encode("utf-8")



import re


def to_databricks_markdown(source: str):
    markdowns_fixed = source.replace("# %% [markdown]", "# %%\n%md")
    blocks_split = re.split(r"# %%\n+", markdowns_fixed)
    converted_list = []
    for line in blocks_split:
        if line.startswith("%md"):
            line = re.sub(r"\n# |\n#", r"\n", line)
            converted_list.append(line)
        else:
            converted_list.append(line)

    return "# %%\n".join(converted_list)


def to_jupyter_markdown(source: str) -> str:
    magic_removed = source.replace("# MAGIC ", "# ")
    magic_removed = magic_removed.replace("# MAGIC", "#")
    empty_lines_removed = magic_removed.replace("# %%\n\n", "# %%\n")
    lines = empty_lines_removed.split("\n")
    processed_file = []
    for index, line in enumerate(lines):
        if line.startswith("# %md"):
            if line.__len__() > 5:
                markdown_separated = "#" + line[5:]
                lines.insert(index + 1, markdown_separated)
            if index > 0 and lines[index - 1] == "# %%":
                previous_value = lines[index - 1]
                previous_value_replaced = previous_value.replace("# %%", "# %% [markdown]")
                processed_file = processed_file[:-1]
                processed_file.append(previous_value_replaced)
        else:
            processed_file.append(line)

    return "\n".join(processed_file)



from dbxdeploy.utils.DatabricksClient import DatabricksClient
from pathlib import PurePosixPath
from dbxdeploy.package.PackageMetadata import PackageMetadata
import re


class RunsGetter:
    def __init__(self, workspace_base_dir: PurePosixPath, dbx_api: DatabricksClient):
        self.__workspace_base_dir = workspace_base_dir
        self.__dbx_api = dbx_api

    def get(self, notebook_path: PurePosixPath, cluster_id: str, package_metadata: PackageMetadata):
        notebook_runs = []

        page = 1
        limit = 50

        while True:
            response = self.__dbx_api.jobs.list_runs(active_only=True, limit=limit, offset=((page - 1) * limit))

            if "runs" in response:
                new_runs = list(filter(lambda run: self.__filter_run(run, notebook_path, cluster_id, package_metadata), response["runs"]))
                notebook_runs = notebook_runs + new_runs

            if response["has_more"] is False:
                break

            page += 1

        return notebook_runs

    def __filter_run(self, run: dict, notebook_path: PurePosixPath, cluster_id: str, package_metadata: PackageMetadata):
        reg_ex = package_metadata.get_notebook_path_reg_ex(self.__workspace_base_dir, notebook_path)

        if "notebook_task" not in run["task"]:
            return False

        return re.match(reg_ex, run["task"]["notebook_task"]["notebook_path"]) and run["cluster_spec"]["existing_cluster_id"] == cluster_id



import re

import databricks_cli.sdk.service as services
from databricks_cli.sdk import ApiClient


def camel_to_snake(name):
    s = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s).lower()


def _get_services():
    for service_name, service in services.__dict__.items():
        if "Service" in service_name:
            snake_name = camel_to_snake(service_name.replace("Service", ""))
            yield service_name, snake_name, service


class DatabricksClient:
    jobs: services.JobsService
    cluster: services.ClusterService
    policy: services.PolicyService
    managed_library: services.ManagedLibraryService
    dbfs: services.DbfsService
    workspace: services.WorkspaceService
    secret: services.SecretService
    groups: services.GroupsService
    token: services.TokenService
    instance_pool: services.InstancePoolService
    delta_pipelines: services.DeltaPipelinesService
    repos: services.ReposService

    def __init__(self, client: ApiClient):
        self.client = client

        for _, snake_name, service in _get_services():
            setattr(self, snake_name, service(self.client))



from pathlib import PurePosixPath


class RelativePathResolver:
    def __init__(
        self,
        base_dir_path: str,
    ):
        self.__base_dir_path = base_dir_path

    def resolve(self, notebook_path: PurePosixPath):
        return notebook_path.relative_to(self.__base_dir_path).with_suffix("")



from dbxdeploy.utils.DatabricksClient import DatabricksClient
from logging import Logger


class ClusterRestarter:
    def __init__(self, logger: Logger, dbx_api: DatabricksClient):
        self.__logger = logger
        self.__dbx_api = dbx_api

    def restart(self, cluster_id: str):
        self.__logger.info("Checking state of cluster {}".format(cluster_id))
        cluster = self.__dbx_api.cluster.get_cluster(cluster_id)

        if cluster["state"] != "PENDING" and cluster["state"] != "RUNNING":
            self.__logger.info("Cannot in {} state, restart not needed".format(cluster["state"]))
            return

        self.__logger.info("Restarting cluster {}".format(cluster_id))
        self.__dbx_api.cluster.restart_cluster(cluster_id)



from pathlib import Path
from pygit2 import Repository, discover_repository, GitError


class CurrentRepositoryFactory:
    def __init__(
        self,
        base_path: Path,
    ):
        self.__base_path = base_path

    def create(self):
        repository_path = discover_repository(self.__base_path.as_posix())

        if not repository_path:
            raise GitError(f'No repository found at "{self.__base_path}" and its parents')

        return Repository(repository_path)



from typing import List
from datetime import datetime
from pathlib import PurePosixPath
from dbxdeploy.package.Dependency import Dependency


class PackageMetadata:
    def __init__(
        self,
        package_name: str,
        package_version: str,
        date_time: datetime,
        random_string: str,
        dependencies: List[Dependency],
    ):
        self.__package_name = package_name
        self.__package_version = f"{package_version}.{date_time.strftime('%Y%m%d%H%M%S%f')}"
        self.__date_time = date_time
        self.__random_string = random_string
        self.__dependencies = dependencies

    @property
    def package_name(self):
        return self.__package_name

    @property
    def package_version(self):
        return self.__package_version

    @property
    def date_time(self):
        return self.__date_time

    @property
    def random_string(self):
        return self.__random_string

    @property
    def dependencies(self):
        return self.__dependencies

    def get_package_filename(self):
        return f"{self.__get_package_name()}-{self.__package_version}-py3-none-any.whl"

    def get_notebook_path_reg_ex(self, workspace_base_dir: PurePosixPath, notebook_path: PurePosixPath) -> str:
        return "^" + str(workspace_base_dir) + "/([^/]+)/" + str(notebook_path) + "$"

    def get_job_run_name(self) -> str:
        return self.__date_time.strftime("%Y-%m-%d_%H-%M-%S") + "_" + self.__random_string

    def get_dependency_by_name(self, dependency_name: str) -> Dependency:
        for dependency in self.__dependencies:
            if dependency.dependency_name == dependency_name:
                return dependency

        raise Exception(f"Dependency {dependency_name} not found")

    def __get_package_name(self):
        return self.__package_name.replace("-", "_")



from logging import Logger
from pathlib import Path
from dbxdeploy.cluster.ClusterRestarter import ClusterRestarter
from dbxdeploy.deploy.CurrentAndReleaseDeployer import CurrentAndReleaseDeployer
from dbxdeploy.job.JobsCreatorAndRunner import JobsCreatorAndRunner
from dbxdeploy.job.JobsDeleter import JobsDeleter
from dbxdeploy.notebook.Notebook import Notebook
from dbxdeploy.notebook.NotebooksLocator import NotebooksLocator
from dbxdeploy.package.PackageMetadataLoader import PackageMetadataLoader
from dbxdeploy.package.PackageDeployer import PackageDeployer
import asyncio
from dbxdeploy.deploy.TargetPathsResolver import TargetPathsResolver


class Releaser:
    def __init__(
        self,
        project_base_dir: Path,
        logger: Logger,
        target_paths_resolver: TargetPathsResolver,
        package_metadata_loader: PackageMetadataLoader,
        current_and_release_deployer: CurrentAndReleaseDeployer,
        package_deployer: PackageDeployer,
        cluster_restarter: ClusterRestarter,
        jobs_deleter: JobsDeleter,
        jobs_creator_and_runner: JobsCreatorAndRunner,
        notebooks_locator: NotebooksLocator,
    ):
        self.__project_base_dir = project_base_dir
        self.__logger = logger
        self.__target_paths_resolver = target_paths_resolver
        self.__package_metadata_loader = package_metadata_loader
        self.__current_and_release_deployer = current_and_release_deployer
        self.__package_deployer = package_deployer
        self.__cluster_restarter = cluster_restarter
        self.__jobs_deleter = jobs_deleter
        self.__jobs_creator_and_runner = jobs_creator_and_runner
        self.__notebooks_locator = notebooks_locator

    async def release(self, cluster_id: str):
        package_metadata = self.__package_metadata_loader.load(self.__project_base_dir)

        loop = asyncio.get_event_loop()

        package_release_future = loop.run_in_executor(None, self.__package_deployer.release, package_metadata)
        dbc_deploy_future = loop.run_in_executor(None, self.__current_and_release_deployer.release, package_metadata)

        await package_release_future
        await dbc_deploy_future

        self.__logger.info("--")

        consumer_notebooks = self.__notebooks_locator.locate_consumers()

        if consumer_notebooks:
            self.__cluster_restarter.restart(cluster_id)

            def create_job_notebook_path(consumer_notebook: Notebook):
                return str(
                    self.__target_paths_resolver.get_workspace_release_path(package_metadata) / consumer_notebook.databricks_relative_path
                )

            consumer_notebooks_release_paths = set(map(create_job_notebook_path, consumer_notebooks))

            self.__jobs_deleter.remove(consumer_notebooks_release_paths)

            self.__logger.info("--")

            self.__jobs_creator_and_runner.create_and_run(consumer_notebooks, cluster_id, package_metadata)

        self.__logger.info("Deployment completed")



from argparse import Namespace, ArgumentParser
from consolebundle.ConsoleCommand import ConsoleCommand
from dbxdeploy.repos.RepoManager import RepoManager


class RepoDeleteCommand(ConsoleCommand):
    def __init__(self, repo_manager: RepoManager):
        self.__repo_manager = repo_manager

    def get_command(self) -> str:
        return "dbx:repo:delete"

    def get_description(self):
        return "Deletes a repository on DBX if available"

    def configure(self, argument_parser: ArgumentParser):
        required_args = argument_parser.add_argument_group("required arguments")
        required_args.add_argument("--repo-url", dest="repo_url", help="Project repo url")
        required_args.add_argument("--repo-name", dest="repo_name", help="Project repo name")

    def run(self, input_args: Namespace):
        if not (input_args.repo_url and input_args.repo_name):
            raise Exception("All arguments are required. Check -h for the list of required arguments.")

        self.__repo_manager.delete(input_args.repo_url, input_args.repo_name)



from dbxdeploy.notebook.NotebooksLocator import NotebooksLocator
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.notebook.NotebooksDeployer import NotebooksDeployer


class CurrentAndReleaseDeployer:
    def __init__(
        self,
        notebooks_locator: NotebooksLocator,
        notebooks_deployer: NotebooksDeployer,
    ):
        self.__notebooks_locator = notebooks_locator
        self.__notebooks_deployer = notebooks_deployer

    def deploy(self, package_metadata: PackageMetadata):
        notebooks = self.__notebooks_locator.locate()

        self.__notebooks_deployer.deploy(package_metadata, notebooks)

    def release(self, package_metadata: PackageMetadata):
        notebooks = self.__notebooks_locator.locate()

        self.__notebooks_deployer.release(package_metadata, notebooks)

    def deploy_only_master_package_notebook(self, package_metadata: PackageMetadata):
        master_package_notebook = self.__notebooks_locator.locate_master_package_notebook()

        self.__notebooks_deployer.deploy_only_master_package_notebook(package_metadata, master_package_notebook)



import asyncio
import sys
from logging import Logger
from argparse import Namespace, ArgumentParser
from pathlib import PurePosixPath, Path
from typing import Optional
from dbxdeploy.deploy.DeployerJobSubmitter import DeployerJobSubmitter
from consolebundle.ConsoleCommand import ConsoleCommand
from dbxdeploy.notebook.RelativePathResolver import RelativePathResolver
from dbxdeploy.notebook.converter.NotebookConverterInterface import NotebookConverterInterface
from dbxdeploy.notebook.converter.UnexpectedSourceException import UnexpectedSourceException
from dbxdeploy.notebook.loader import load_notebook


class DeployJobSubmitCommand(ConsoleCommand):
    def __init__(
        self,
        cluster_id: Optional[str],
        logger: Logger,
        notebook_converter: NotebookConverterInterface,
        deployer_job_submitter: DeployerJobSubmitter,
        relative_path_resolver: RelativePathResolver,
    ):
        self.__cluster_id = cluster_id
        self.__logger = logger
        self.__notebook_converter = notebook_converter
        self.__deployer_job_submitter = deployer_job_submitter
        self.__relative_path_resolver = relative_path_resolver

    def configure(self, argument_parser: ArgumentParser):
        argument_parser.add_argument(dest="notebook_path", help="Databricks notebook path relative to project root")

        # deprecated, will be removed in 2.0; use CLI argument to defined cluster_id instead
        if self.__cluster_id is None:
            argument_parser.add_argument(dest="cluster_id", help="Cluster to submit job to")

    def get_command(self) -> str:
        return "dbx:deploy-submit-job"

    def get_description(self):
        return "Deploy to DBX and submit selected notebook as job"

    def run(self, input_args: Namespace):
        relative_notebook_path = PurePosixPath(input_args.notebook_path)
        notebook_path = Path().cwd().joinpath(relative_notebook_path)

        source = load_notebook(notebook_path)

        try:
            self.__notebook_converter.validate_source(source)
        except UnexpectedSourceException:
            self.__logger.error("Only valid Databricks notebooks can be submitted as Databricks job")
            sys.exit(1)

        relative_notebook_path = self.__relative_path_resolver.resolve(relative_notebook_path)

        cluster_id = self.__cluster_id if self.__cluster_id is not None else input_args.cluster_id

        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.__deployer_job_submitter.deploy_and_submit_job(relative_notebook_path, cluster_id))



import unittest
from pathlib import PurePosixPath
from dbxdeploy.dbc.PathsPreparer import PathsPreparer


class PathsPreparerTest(unittest.TestCase):
    def setUp(self):
        self.__paths_preparer = PathsPreparer()

    def test_basic(self):
        paths = [
            PurePosixPath("DataSenticsLib/Ahoj/notebook_spark2.ipynb"),
            PurePosixPath("DataSenticsLib/Example/notebook_spark.ipynb"),
            PurePosixPath("DataSenticsLib/Example/notebook_spark3.ipynb"),
            PurePosixPath("DataSenticsLib/Example/SubExample/notebook_pandas.ipynb"),
        ]

        unique_paths = sorted(self.__paths_preparer.prepare(paths, "root_ignored_path"))

        self.assertListEqual(
            [
                "root_ignored_path",
                "root_ignored_path/DataSenticsLib",
                "root_ignored_path/DataSenticsLib/Ahoj",
                "root_ignored_path/DataSenticsLib/Example",
                "root_ignored_path/DataSenticsLib/Example/SubExample",
            ],
            unique_paths,
        )


if __name__ == "__main__":
    unittest.main()



from base64 import b64encode
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from dbxdeploy.package.PackageUploaderInterface import PackageUploaderInterface
from requests.exceptions import HTTPError


class DbfsFileUploader(PackageUploaderInterface):
    def __init__(self, dbx_api: DatabricksClient):
        self.__dbx_api = dbx_api

    def upload(self, content: bytes, file_path: str, overwrite: bool = False):
        self.__streaming_upload(content, file_path, overwrite)

    def exists(self, file_path: str):
        try:
            self.__dbx_api.dbfs.get_status(file_path)

        except HTTPError as ex:
            if ex.response.status_code != 404:
                raise

            return False

        return True

    def __streaming_upload(self, content: bytes, file_path: str, overwrite: bool = False):
        response = self.__dbx_api.dbfs.create(file_path, overwrite=overwrite)
        handle = response["handle"]

        chunk_size = int(0.5 * 1024 * 1024)  # 0.5MiB

        for chunk in self.__chunked(content, chunk_size):
            chunk_encoded = b64encode(chunk).decode()

            self.__dbx_api.dbfs.add_block(handle, chunk_encoded)

        self.__dbx_api.dbfs.close(handle)

    def __chunked(self, source, size):
        for i in range(0, len(source), size):
            yield source[i : i + size]  # noqa: 5203



from pathlib import PurePosixPath
from typing import List


class PathsPreparer:
    def prepare(self, databricks_relative_paths: List[PurePosixPath], root_ignored_path_name: str):
        def create_notebook_path(databricks_relative_path: PurePosixPath) -> tuple:
            return (root_ignored_path_name,) + databricks_relative_path.parts[0:-1]

        paths = list(map(create_notebook_path, databricks_relative_paths))
        paths = list(set(paths))

        for path in paths:
            for i in range(len(path) - 1):
                x = path[0 : i + 1]  # noqa: 5203
                paths.append(x)

        unique_paths = list(set(paths))
        unique_paths = sorted(unique_paths, key=len)

        return list(map("/".join, unique_paths))



import asyncio
from argparse import Namespace, ArgumentParser
from logging import Logger
from pathlib import PurePosixPath
from typing import Optional
from dbxdeploy.deploy.Releaser import Releaser
from consolebundle.ConsoleCommand import ConsoleCommand


class ReleaseCommand(ConsoleCommand):
    def __init__(self, cluster_id: Optional[str], dbx_host: str, workspace_base_dir: PurePosixPath, logger: Logger, releaser: Releaser):
        self.__cluster_id = cluster_id
        self.__dbx_host = dbx_host
        self.__workspace_base_dir = workspace_base_dir
        self.__logger = logger
        self.__releaser = releaser

    def get_command(self) -> str:
        return "dbx:release"

    def get_description(self):
        return "Release the app to the production environment"

    def configure(self, argument_parser: ArgumentParser):
        # deprecated, will be removed in 2.0; use CLI argument to defined cluster_id instead
        if self.__cluster_id is None:
            argument_parser.add_argument(dest="cluster_id", help="Cluster to be restarted before release finishes")

    def run(self, input_args: Namespace):
        cluster_id = self.__cluster_id if self.__cluster_id is not None else input_args.cluster_id

        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.__releaser.release(cluster_id))

        self.__logger.info(f"Released to {self.__dbx_host}/#workspace{self.__workspace_base_dir}")



from pathlib import Path, PurePosixPath
from typing import List
from dbxdeploy.notebook.Notebook import Notebook
from dbxdeploy.notebook.RelativePathResolver import RelativePathResolver


class NotebooksLocator:
    def __init__(
        self,
        project_base_dir: Path,
        relative_base_dir: str,
        paths_patterns: list,
        consumer_paths_patterns: list,
        master_package_notebook_path: str,
        relative_path_resolver: RelativePathResolver,
    ):
        self.__project_base_dir = project_base_dir
        self.__relative_base_dir = relative_base_dir
        self.__paths_patterns = paths_patterns
        self.__consumer_paths_patterns = consumer_paths_patterns
        self.__master_package_notebook_path = master_package_notebook_path
        self.__relative_path_resolver = relative_path_resolver

    def locate(self) -> List[Notebook]:
        return self.__locate(self.__paths_patterns)

    def locate_consumers(self):
        return self.__locate(self.__consumer_paths_patterns)

    def locate_master_package_notebook(self) -> Notebook:
        notebook_path = self.__project_base_dir.joinpath(self.__master_package_notebook_path)
        pure_posix_path = PurePosixPath(notebook_path.relative_to(self.__project_base_dir).as_posix())

        if not notebook_path.is_file():
            raise Exception(f"Notebook at {notebook_path} not found")

        return Notebook(
            notebook_path, notebook_path.relative_to(self.__project_base_dir), self.__relative_path_resolver.resolve(pure_posix_path)
        )

    def __locate(self, paths_patterns: list):
        def create_notebook(path: Path):
            pure_posix_path = PurePosixPath(path.relative_to(self.__project_base_dir).as_posix())
            return Notebook(path, path.relative_to(self.__project_base_dir), self.__relative_path_resolver.resolve(pure_posix_path))

        base_dir = self.__project_base_dir.joinpath(self.__relative_base_dir)

        files_grabbed = []

        for path_pattern in paths_patterns:
            cleaned = [file for file in base_dir.glob(path_pattern) if not PurePosixPath(file).match(".ipynb_checkpoints/*.py")]
            files_grabbed.extend(cleaned)

        return list(map(create_notebook, files_grabbed))



import unittest
from dbxdeploy.notebook.converter import empty_lines_remover


class empty_lines_remover_test(unittest.TestCase):  # noqa: N801
    def test_forced_end_file_new_line(self):
        code = empty_lines_remover.remove(
            "\n".join(
                (
                    "# MAGIC %md #### Something",
                    "",
                    "# COMMAND ----------",
                    "",
                    "",
                    "@transformation(read_csv_mask_usage, display=True)",
                    "def add_column_insert_ts(df: DataFrame):",
                    '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                    "",
                    "",
                    "# COMMAND ----------",
                    "",
                    '# MAGIC %md #### Saving results to fresh table"',
                )
            )
        )

        expected_result = "\n".join(
            (
                "# MAGIC %md #### Something",
                "",
                "# COMMAND ----------",
                "",
                "@transformation(read_csv_mask_usage, display=True)",
                "def add_column_insert_ts(df: DataFrame):",
                '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                "",
                "# COMMAND ----------",
                "",
                '# MAGIC %md #### Saving results to fresh table"',
            )
        )

        self.assertEqual(expected_result, code)


if __name__ == "__main__":
    unittest.main()



import re
from pathlib import Path
from typing import Match
from tomlkit import inline_table
from dbxdeploy.package.requirements_regex import (
    CLASSIC_LINE_REGEX,
    GIT_LINE_REGEX,
    GIT_OLD_LINE1_REGEX,
    GIT_OLD_LINE2_REGEX,
    LOCAL_FILE_LINE_REGEX,
)


class RequirementsLineConverter:
    def parse(self, line: str):
        matches = re.match(CLASSIC_LINE_REGEX, line)

        if matches:
            return self.__parse_line(matches)

        matches = re.match(GIT_OLD_LINE1_REGEX, line)

        if matches:
            return self.__parse_git_old_line(matches)

        matches = re.match(GIT_OLD_LINE2_REGEX, line)

        if matches:
            return self.__parse_git_old_line(matches)

        matches = re.match(GIT_LINE_REGEX, line)

        if matches:
            return self.__parse_git_line(matches)

        matches = re.match(LOCAL_FILE_LINE_REGEX, line)

        if matches:
            return self.__parse_local_file_line(matches)

        raise Exception("Invalid requirements.txt line: {}".format(line))

    def __parse_line(self, matches: Match[str]):
        if matches.group(3) is None:
            return [matches.group(1), matches.group(2).strip()]

        it = inline_table()
        it.append("version", matches.group(2))
        it.append("markers", matches.group(3))

        return [matches.group(1), it]

    def __parse_git_old_line(self, matches: Match[str]):
        it = inline_table()
        it.append("git", matches.group(1))
        it.append("rev", matches.group(2))

        return [matches.group(3), it]

    def __parse_git_line(self, matches: Match[str]):
        it = inline_table()
        it.append("git", matches.group(2))
        it.append("rev", matches.group(3))

        return [matches.group(1), it]

    def __parse_local_file_line(self, matches: Match[str]):
        file_path = Path(matches.group(2))
        name = matches.group(1)
        version = file_path.name.split("-")[1]
        markers = matches.group(3)

        if markers is None:
            return [name, version]

        it = inline_table()
        it.append("version", version)
        it.append("markers", markers)

        return [name, it]



import asyncio
from argparse import Namespace
from logging import Logger
from pathlib import PurePosixPath
from dbxdeploy.deploy.Deployer import Deployer
from consolebundle.ConsoleCommand import ConsoleCommand


class DeployCommand(ConsoleCommand):
    def __init__(self, dbx_host: str, workspace_base_dir: PurePosixPath, logger: Logger, deployer: Deployer):
        self.__dbx_host = dbx_host
        self.__workspace_base_dir = workspace_base_dir
        self.__logger = logger
        self.__deployer = deployer

    def get_command(self) -> str:
        return "dbx:deploy"

    def get_description(self):
        return "Deploy notebooks and master package to DBX"

    def run(self, input_args: Namespace):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.__deployer.deploy())

        self.__logger.info(f"Deployed to {self.__dbx_host}/#workspace{self.__workspace_base_dir}")



import re
from typing import List


class CellsExtractor:
    def extract(self, original_script: str, separator_regexp: str) -> List[dict]:
        def remove_ending_spaces(cell: str):
            return re.sub(r"\n+$", "", cell)

        raw_cells = re.split(separator_regexp, original_script)
        raw_cells = list(filter(lambda raw_cell: raw_cell != "", raw_cells))
        raw_cells = list(map(remove_ending_spaces, raw_cells))

        return list(map(lambda raw_cell: {"source": raw_cell, "cell_type": "code"}, raw_cells))



from abc import ABC, abstractmethod


class NotebookConverterInterface(ABC):
    @abstractmethod
    def validate_source(self, source: str):
        pass

    @abstractmethod
    def from_dbc_notebook(self, content: dict) -> str:
        pass

    @abstractmethod
    def to_dbc_notebook(self, notebook_name: str, source: str, package_file_path: str, dependencies_dir_path: str) -> str:
        pass

    @abstractmethod
    def to_workspace_import_notebook(self, source: str, package_file_path: str, dependencies_dir_path: str) -> str:
        pass



class Dependency:
    def __init__(
        self,
        dependency_name: str,
        dependency_version: str,
    ):

        self.__dependency_name = dependency_name
        self.__dependency_version = dependency_version

    @property
    def dependency_name(self):
        return self.__dependency_name

    @property
    def dependency_version(self):
        return self.__dependency_version



import re
from dbxdeploy.package.PackageInstaller import PackageInstaller


class CommandConverter:
    def __init__(
        self,
        package_installer: PackageInstaller,
    ):
        self.__package_installer = package_installer

    def convert(self, command: dict):
        if self.__package_installer.is_package_install_command(command["command"]):
            return self.__process_title("# MAGIC %install_master_package_whl", command)

        magic_command = self.__detect_magic_command(command["command"])

        if magic_command:
            command_code = "# MAGIC " + command["command"].replace("\n", "\n# MAGIC ")

            return self.__process_title(command_code, command)

        return self.__process_title(command["command"], command)

    def __detect_magic_command(self, command_code: str):
        matches = re.match(r"^(%[a-z_]+)[\s]", command_code)

        if not matches:
            return None

        return matches.group(1)

    def __process_title(self, command_code: str, orig_command: dict):
        if not orig_command["commandTitle"]:
            return command_code

        show_command_title_string = "1" if orig_command["showCommandTitle"] is True else "0"

        return f'# DBTITLE {show_command_title_string},{orig_command["commandTitle"]}\n{command_code}'



import unittest
from dbxdeploy.package.RequirementsLineConverter import RequirementsLineConverter


class RequirementsLineConverterTest(unittest.TestCase):
    def setUp(self):
        self.__requirements_line_converter = RequirementsLineConverter()

    def test_basic(self):
        result = self.__requirements_line_converter.parse("appnope==0.1.0")

        self.assertEqual(["appnope", "0.1.0"], result)

    def test_with_sys_platform(self):
        result = self.__requirements_line_converter.parse('appnope==0.1.0; sys_platform == "darwin"')

        self.assertEqual(["appnope", {"version": "0.1.0", "markers": 'sys_platform == "darwin"'}], result)

    def test_multiple_conditions(self):
        result = self.__requirements_line_converter.parse(
            'pywin32==227; sys_platform == "win32" and python_version >= "3.6" or sys_platform == "win32"'
        )

        self.assertEqual(
            ["pywin32", {"version": "227", "markers": 'sys_platform == "win32" and python_version >= "3.6" or sys_platform == "win32"'}],
            result,
        )

    def test_git_ssh(self):
        result = self.__requirements_line_converter.parse("-e git+git@github.com:myaccount/myrepo.git@master#egg=my-package-name")

        self.assertEqual(["my-package-name", {"git": "git@github.com:myaccount/myrepo.git", "rev": "master"}], result)

    def test_git_https_old(self):
        result = self.__requirements_line_converter.parse(
            "-e git+https://github.com/myaccount/myrepo.git@82f1bb9de9665918c262465e72e872e4f744112c#egg=my-package-name"
        )

        self.assertEqual(
            ["my-package-name", {"git": "https://github.com/myaccount/myrepo.git", "rev": "82f1bb9de9665918c262465e72e872e4f744112c"}],
            result,
        )

    def test_git_https_with_token_old(self):
        result = self.__requirements_line_converter.parse(
            "-e git+https://sometoken@github.com/myaccount/myrepo.git@82f1bb9de9665918c262465e72e872e4f744112c#egg=my-package-name"
        )

        self.assertEqual(
            [
                "my-package-name",
                {"git": "https://sometoken@github.com/myaccount/myrepo.git", "rev": "82f1bb9de9665918c262465e72e872e4f744112c"},
            ],
            result,
        )

    def test_git_https(self):
        result = self.__requirements_line_converter.parse(
            "my-package-name @ git+https://sometoken@github.com/myaccount/myrepo.git@master#egg=my-package-name"
        )

        self.assertEqual(["my-package-name", {"git": "https://sometoken@github.com/myaccount/myrepo.git", "rev": "master"}], result)

    def test_git_https_no_egg(self):
        result = self.__requirements_line_converter.parse("my-package-name @ git+https://sometoken@github.com/myaccount/myrepo.git@master")

        self.assertEqual(["my-package-name", {"git": "https://sometoken@github.com/myaccount/myrepo.git", "rev": "master"}], result)

    def test_git_https_with_token(self):
        result = self.__requirements_line_converter.parse(
            "my-package-name @ git+https://github.com/myaccount/myrepo.git@master#egg=my-package-name"
        )

        self.assertEqual(["my-package-name", {"git": "https://github.com/myaccount/myrepo.git", "rev": "master"}], result)

    def test_git_https_with_token_no_egg(self):
        result = self.__requirements_line_converter.parse("my-package-name @ git+https://github.com/myaccount/myrepo.git@master")

        self.assertEqual(["my-package-name", {"git": "https://github.com/myaccount/myrepo.git", "rev": "master"}], result)

    def test_local_windows_file(self):
        result = self.__requirements_line_converter.parse(
            "typed-ast @ file:///C:/dependencies/local/typed_ast-1.4.2-cp37-cp37m-win_amd64.whl"
        )

        self.assertEqual(["typed-ast", "1.4.2"], result)

    def test_local_windows_file_with_markers(self):
        result = self.__requirements_line_converter.parse(
            'zipp @ file:///C:/dependencies/local/zipp-3.4.0-py3-none-any.whl; python_version >= "3.6"'
        )

        self.assertEqual(["zipp", {"version": "3.4.0", "markers": 'python_version >= "3.6"'}], result)

    def test_local_linux_file(self):
        result = self.__requirements_line_converter.parse(
            "typed-ast @ file:///home/user/work/project/dependencies/local/typed_ast-1.4.2-cp37-cp37m-win_amd64.whl"
        )

        self.assertEqual(["typed-ast", "1.4.2"], result)

    def test_local_linux_file_with_markers(self):
        result = self.__requirements_line_converter.parse(
            'zipp @ file:///home/user/work/project/local/zipp-3.4.0-py3-none-any.whl; python_version >= "3.6"'
        )

        self.assertEqual(["zipp", {"version": "3.4.0", "markers": 'python_version >= "3.6"'}], result)


if __name__ == "__main__":
    unittest.main()



from io import BytesIO
import zipfile


class DbcFilesHandler:
    def handle(self, dbc_content: bytes, callback: callable):
        buffer = BytesIO()
        buffer.write(dbc_content)

        zip_file = zipfile.ZipFile(buffer, "r", zipfile.ZIP_DEFLATED)

        for file in zip_file.filelist:
            callback(zip_file, file)

        zip_file.close()
        buffer.seek(0)



class RequirementsConfig:
    def __init__(
        self,
        include_dev_dependencies=False,
        include_credentials=False,
        exclude_index_info=False,
        exclude_file_dependencies=False,
        redact_credentials=False,
    ):
        self.__include_dev_dependencies = include_dev_dependencies
        self.__include_credentials = include_credentials
        self.__exclude_index_info = exclude_index_info
        self.__exclude_file_dependencies = exclude_file_dependencies
        self.__redact_credentials = redact_credentials

    def include_dev_dependencies(self):
        self.__include_dev_dependencies = True

    def include_credentials(self):
        self.__include_credentials = True

    def exclude_index_info(self):
        self.__exclude_index_info = True

    def exclude_file_dependencies(self):
        self.__exclude_file_dependencies = True

    def redact_credentials(self):
        self.__redact_credentials = True

    @property
    def should_include_dev_dependencies(self):
        return self.__include_dev_dependencies

    @property
    def should_include_credentials(self):
        return self.__include_credentials

    @property
    def should_exclude_index_info(self):
        return self.__exclude_index_info

    @property
    def should_exclude_file_dependencies(self):
        return self.__exclude_file_dependencies

    @property
    def should_redact_credentials(self):
        return self.__redact_credentials



from pathlib import Path
from pathlib import PurePosixPath
from typing import List
from base64 import b64decode
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from requests.exceptions import HTTPError


class DbfsFileDownloader:
    def __init__(self, dbx_api: DatabricksClient):
        self.__dbx_api = dbx_api

    def download(self, dbfs_path: PurePosixPath):
        return self.__streaming_download(dbfs_path)

    def exists(self, dbfs_path: PurePosixPath):
        try:
            self.__dbx_api.dbfs.get_status(dbfs_path)

        except HTTPError as ex:
            if ex.response.status_code != 404:
                raise

            return False

        return True

    def download_directory(self, dbfs_dir: PurePosixPath, local_dir: Path, overwrite: bool = False):
        local_dir.mkdir(exist_ok=True)

        remote_file_paths = self.list_directory_recursive(dbfs_dir, only_files=True)
        local_file_paths = [local_dir.joinpath(file.relative_to(dbfs_dir)) for file in remote_file_paths]

        if not overwrite:
            for local_file_path in local_file_paths:
                if local_dir.joinpath(local_file_path).exists():
                    raise Exception(f"File {local_file_path} already exists")

        for remote_file_path, local_file_path in zip(remote_file_paths, local_file_paths):
            content = self.download(remote_file_path)

            local_file_path.parent.mkdir(parents=True, exist_ok=True)

            with local_file_path.open("wb") as f:
                f.write(content)

    def list_directory_recursive(self, dbfs_dir: PurePosixPath, only_files: bool = False) -> List[PurePosixPath]:
        def recursive_walk(dbfs_dir: PurePosixPath, files: List[PurePosixPath], only_files: bool = False):
            response = self.__dbx_api.dbfs.list(str(dbfs_dir))

            if "files" in response:
                for file in response["files"]:
                    if file["is_dir"]:
                        recursive_walk(file["path"], files, only_files)

                    # always add file
                    if not file["is_dir"]:
                        files.append(PurePosixPath("dbfs:" + file["path"]))

                    # add dir only if only_files=False
                    if file["is_dir"] and not only_files:
                        files.append(PurePosixPath("dbfs:" + file["path"]))

        if not self.exists(dbfs_dir):
            raise Exception(f"Directory at {dbfs_dir} does not exists")

        files = []
        recursive_walk(dbfs_dir, files, only_files)

        return files

    def __streaming_download(self, dbfs_path: PurePosixPath) -> bytes:
        file_info = self.__dbx_api.dbfs.get_status(dbfs_path)

        chunk_size = int(0.5 * 1024 * 1024)  # 0.5MiB
        file_size = file_info["file_size"]
        offset = 0
        data = bytes()

        while offset < file_size:
            response = self.__dbx_api.dbfs.read(dbfs_path, offset, chunk_size)
            offset += response["bytes_read"]
            data += b64decode(response["data"])

        return data



from pathlib import PurePosixPath
from typing import List
from requests.exceptions import HTTPError
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from dbxdeploy.dbc.DbcCreator import DbcCreator
from dbxdeploy.dbc.DbcUploader import DbcUploader
from dbxdeploy.notebook.CurrentDirectoryUpdater import CurrentDirectoryUpdater
from dbxdeploy.notebook.Notebook import Notebook
from dbxdeploy.package.PackageMetadata import PackageMetadata
from logging import Logger
from dbxdeploy.deploy.TargetPathsResolver import TargetPathsResolver


class NotebooksDeployer:
    def __init__(
        self,
        workspace_base_dir: PurePosixPath,
        logger: Logger,
        dbx_api: DatabricksClient,
        dbc_creator: DbcCreator,
        dbc_uploader: DbcUploader,
        current_directory_updater: CurrentDirectoryUpdater,
        target_paths_resolver: TargetPathsResolver,
    ):
        self.__workspace_base_dir = workspace_base_dir
        self.__logger = logger
        self.__dbx_api = dbx_api
        self.__dbc_creator = dbc_creator
        self.__dbc_uploader = dbc_uploader
        self.__current_directory_updater = current_directory_updater
        self.__target_paths_resolver = target_paths_resolver

    def deploy(self, package_metadata: PackageMetadata, notebooks: List[Notebook]):
        package_path = self.__target_paths_resolver.get_package_upload_path_for_deploy(package_metadata)
        dependencies_dir = self.__target_paths_resolver.get_dependencies_upload_dir_for_deploy(package_metadata)

        self.__logger.info(f"All packages released, updating {self.__workspace_base_dir}")
        self.__current_directory_updater.update(notebooks, self.__workspace_base_dir, package_path, dependencies_dir)

    def deploy_only_master_package_notebook(self, package_metadata: PackageMetadata, notebook: Notebook):
        package_path = self.__target_paths_resolver.get_package_upload_path_for_deploy(package_metadata)
        dependencies_dir = self.__target_paths_resolver.get_dependencies_upload_dir_for_deploy(package_metadata)

        self.__logger.info(f"All packages released, updating {self.__workspace_base_dir}")
        self.__current_directory_updater.update_only_master_package_notebook(
            notebook, self.__workspace_base_dir, package_path, dependencies_dir
        )

    def release(self, package_metadata: PackageMetadata, notebooks: List[Notebook]):
        package_path = self.__target_paths_resolver.get_package_upload_path_for_release(package_metadata)
        dependencies_dir = self.__target_paths_resolver.get_dependencies_upload_dir_for_release(package_metadata)

        self.__logger.info("Building notebooks package (DBC)")
        dbc_content = self.__dbc_creator.create(notebooks, package_path, dependencies_dir)

        _current_path = self.__target_paths_resolver.get_workspace_current_path(package_metadata)
        release_path = self.__target_paths_resolver.get_workspace_release_path(package_metadata)

        self.__logger.info(f"Uploading notebooks package to {release_path}")
        self.__dbc_uploader.upload(dbc_content, release_path)

        self.__logger.info(f"Cleaning up {_current_path} if exists")

        try:
            self.__dbx_api.workspace.delete(str(_current_path), recursive=True)
        except HTTPError as e:
            if e.response.status_code != 404:
                raise

        self.__logger.info(f"Uploading notebooks package to {_current_path}")
        self.__dbc_uploader.upload(dbc_content, _current_path)



import unittest
from dbxdeploy.notebook.converter import markdowns_converter


class markdowns_converter_test(unittest.TestCase):  # noqa: N801
    def test_to_databricks_markdown_converter(self):
        code = markdowns_converter.to_databricks_markdown(
            "\n".join(
                (
                    "# %% [markdown]",
                    "# ##TEST",
                    "",
                    "# %%",
                    "@transformation(read_csv_mask_usage, display=True)",
                    "def add_column_insert_ts(df: DataFrame):",
                    '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                    "",
                    "# %% [markdown]",
                    '# #### Saving results to fresh table"',
                )
            )
        )

        expected_result = "\n".join(
            (
                "# %%",
                "%md",
                "##TEST",
                "",
                "# %%",
                "@transformation(read_csv_mask_usage, display=True)",
                "def add_column_insert_ts(df: DataFrame):",
                '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                "",
                "# %%",
                "%md",
                '#### Saving results to fresh table"',
            )
        )

        self.assertEqual(expected_result, code)

    def test_to_jupyter_md_on_first_line(self):
        code = markdowns_converter.to_jupyter_markdown(
            "\n".join(
                (
                    "# %%",
                    "# %md",
                    "# ##TEST",
                    "",
                    "# %%",
                    "@transformation(read_csv_mask_usage, display=True)",
                    "def add_column_insert_ts(df: DataFrame):",
                    '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                    "",
                    "# %%",
                    "# %md",
                    '# #### Saving results to fresh table"',
                )
            )
        )

        expected_result = "\n".join(
            (
                "# %% [markdown]",
                "# ##TEST",
                "",
                "# %%",
                "@transformation(read_csv_mask_usage, display=True)",
                "def add_column_insert_ts(df: DataFrame):",
                '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                "",
                "# %% [markdown]",
                '# #### Saving results to fresh table"',
            )
        )

        self.assertEqual(expected_result, code)

    def test_to_jupyter_md_on_second_line(self):
        code = markdowns_converter.to_jupyter_markdown(
            "\n".join(
                (
                    "# %%",
                    "",
                    "# %md",
                    "# ##TEST",
                    "",
                    "# %%",
                    "@transformation(read_csv_mask_usage, display=True)",
                    "def add_column_insert_ts(df: DataFrame):",
                    '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                    "",
                    "# %%",
                    "",
                    "# %md",
                    '# #### Saving results to fresh table"',
                )
            )
        )

        expected_result = "\n".join(
            (
                "# %% [markdown]",
                "# ##TEST",
                "",
                "# %%",
                "@transformation(read_csv_mask_usage, display=True)",
                "def add_column_insert_ts(df: DataFrame):",
                '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                "",
                "# %% [markdown]",
                '# #### Saving results to fresh table"',
            )
        )

        self.assertEqual(expected_result, code)

    def test_to_jupyter_md_with_comment_on_one_line(self):
        code = markdowns_converter.to_jupyter_markdown(
            "\n".join(
                (
                    "# %%",
                    "",
                    "# %md ##TEST",
                    "",
                    "# %%",
                    "@transformation(read_csv_mask_usage, display=True)",
                    "def add_column_insert_ts(df: DataFrame):",
                    '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                    "",
                    "# %%",
                    "",
                    "# %md #### Saving results to fresh table",
                )
            )
        )

        expected_result = "\n".join(
            (
                "# %% [markdown]",
                "# ##TEST",
                "",
                "# %%",
                "@transformation(read_csv_mask_usage, display=True)",
                "def add_column_insert_ts(df: DataFrame):",
                '    return df.withColumn("INSERT_TS", f.lit(datetime.now()))',
                "",
                "# %% [markdown]",
                "# #### Saving results to fresh table",
            )
        )

        self.assertEqual(expected_result, code)


if __name__ == "__main__":
    unittest.main()



from typing import List, Dict

import tomlkit
from pyfonycore.pyproject import read, get_path
from pathlib import Path

from tomlkit.toml_document import TOMLDocument


class CodeStyleLoader:
    def __init__(
        self,
        package_base_dir: Path,
    ):
        self.__package_base_dir = package_base_dir

    def get_codestyle_install_command(self) -> str:
        versions = self.__get_versions(["flake8", "pycodestyle-magic"])

        return (
            f"# {self.get_setup_command()}\n"
            "import IPython  # noqa E402\n"
            f"IPython.get_ipython().run_line_magic('pip', 'install flake8=={versions['flake8']} pycodestyle_magic=={versions['pycodestyle-magic']}')\n"
        )

    def get_codestyle_setup_command(self) -> str:
        toml_doc = self.__load_pyproject_toml()
        flake8_cmd = toml_doc["tool"]["poe"]["tasks"]["flake8"]
        flake8_options = str(flake8_cmd).split()
        flake8_options = " ".join([f"--{opt[2:].replace('=', ' ').replace('-', '_')}" for opt in flake8_options if opt.startswith("--")])

        return (
            "import IPython  # noqa E402\n"
            f"ipy = IPython.get_ipython()\n"
            "ipy.run_line_magic('load_ext', 'pycodestyle_magic')\n"
            f"ipy.run_line_magic('flake8_on', '{flake8_options}')"
        )

    def get_setup_command(self) -> str:
        return "%flake8_setup"

    def __load_pyproject_toml(self) -> TOMLDocument:
        return read(get_path(self.__package_base_dir))

    def __get_versions(self, packages: List[str]) -> Dict[str, str]:
        lockfile_path = self.__package_base_dir.joinpath("poetry.lock")
        with lockfile_path.open("r") as f:
            config = tomlkit.parse(f.read())

        return {
            package["name"]: package["version"]
            for package in config["package"]
            if package["category"] == "dev" and package["name"] in packages
        }



from dbxdeploy.job.NotebookKiller import NotebookKiller
from dbxdeploy.notebook.NotebooksLocator import NotebooksLocator
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.package.PackageMetadataLoader import PackageMetadataLoader
from dbxdeploy.package.PackageDeployer import PackageDeployer
from dbxdeploy.notebook.NotebooksDeployer import NotebooksDeployer
from dbxdeploy.job.JobSubmitter import JobSubmitter
from pathlib import Path, PurePosixPath
import asyncio


class DeployerJobSubmitter:
    def __init__(
        self,
        project_base_dir: Path,
        package_metadata_loader: PackageMetadataLoader,
        notebook_killer: NotebookKiller,
        notebooks_deployer: NotebooksDeployer,
        package_deployer: PackageDeployer,
        job_submitter: JobSubmitter,
        notebooks_locator: NotebooksLocator,
    ):
        self.__project_base_dir = project_base_dir
        self.__package_metadata_loader = package_metadata_loader
        self.__notebook_killer = notebook_killer
        self.__notebooks_deployer = notebooks_deployer
        self.__package_deployer = package_deployer
        self.__job_submitter = job_submitter
        self.__notebooks_locator = notebooks_locator

    async def deploy_and_submit_job(self, notebook_path: PurePosixPath, cluster_id: str):
        package_metadata = self.__package_metadata_loader.load(self.__project_base_dir)

        def deploy_root(package_metadata: PackageMetadata):
            notebooks = self.__notebooks_locator.locate()
            self.__notebooks_deployer.deploy(package_metadata, notebooks)

        loop = asyncio.get_event_loop()

        notebook_killer_future = loop.run_in_executor(
            None, self.__notebook_killer.kill_if_running, notebook_path, cluster_id, package_metadata
        )
        package_deploy_future = loop.run_in_executor(None, self.__package_deployer.deploy, package_metadata)
        dbc_deploy_future = loop.run_in_executor(None, deploy_root, package_metadata)

        await notebook_killer_future
        await package_deploy_future
        await dbc_deploy_future

        self.__job_submitter.submit(notebook_path, cluster_id, package_metadata)



import random
import string


class RandomStringGenerator:
    def generate(self, string_length: int):
        letters = string.ascii_lowercase
        return "".join(random.choice(letters) for i in range(string_length))



import time
from pathlib import Path
from pathlib import PurePosixPath
from logging import Logger
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.package.RequirementsGenerator import RequirementsGenerator
from dbxdeploy.package.RequirementsConfig import RequirementsConfig
from dbxdeploy.dbfs.DbfsFileUploader import DbfsFileUploader
from dbxdeploy.dbfs.DbfsFileDownloader import DbfsFileDownloader


class DependencyBuilder:
    def __init__(
        self,
        project_base_dir: Path,
        logger: Logger,
        dbx_api: DatabricksClient,
        dbfs_file_uploader: DbfsFileUploader,
        dbfs_file_downloader: DbfsFileDownloader,
        job_cluster_definition: dict,
        requirements_generator: RequirementsGenerator,
    ):
        self.__project_base_dir = project_base_dir
        self.__logger = logger
        self.__dbx_api = dbx_api
        self.__dbfs_file_uploader = dbfs_file_uploader
        self.__dbfs_file_downloader = dbfs_file_downloader
        self.__job_cluster_definition = job_cluster_definition
        self.__requirements_generator = requirements_generator

    def build(self, base_path: Path, package_metadata: PackageMetadata, dev_dependencies: bool = False):
        self.__logger.info("Preparing packages build script...")

        dbfs_build_dir = PurePosixPath(f"dbfs:/tmp/DependencyBuild/{package_metadata.get_job_run_name()}")
        dbfs_script_path = dbfs_build_dir.joinpath("download_dependencies.py")
        dbfs_requirements_path = dbfs_build_dir.joinpath("requirements.txt")
        dbfs_dependencies_dir = dbfs_build_dir.joinpath("dependencies")
        local_dependencies_dir = base_path.joinpath("dependencies")
        unix_dependencies_dir = "/dbfs/" + dbfs_dependencies_dir.as_posix().lstrip("dbfs:/")
        unix_requirements_path = "/dbfs/" + dbfs_requirements_path.as_posix().lstrip("dbfs:/")

        requirements_config = RequirementsConfig()

        if dev_dependencies:
            requirements_config.include_dev_dependencies()

        requirements_config.include_credentials()
        requirements_config.exclude_file_dependencies()

        requirements_txt = self.__requirements_generator.generate(requirements_config)

        requirements_config.redact_credentials()

        requirements_txt_redacted = self.__requirements_generator.generate(requirements_config)

        script = (
            f"import sys, subprocess\n"
            f"subprocess.run(\n"
            f'    [sys.executable, "-m", "pip", "wheel", "-w", "{unix_dependencies_dir}", "-r", "{unix_requirements_path}"],\n'
            f'    capture_output=False, encoding="utf-8"\n'
            f")\n"
        )

        self.__logger.info("Uploading build script to dbfs...")
        self.__dbfs_file_uploader.upload(script.encode("utf-8"), str(dbfs_script_path))
        self.__dbfs_file_uploader.upload(requirements_txt.encode("utf-8"), str(dbfs_requirements_path))
        self.__logger.info("Build Script uploaded")

        self.__logger.info("Building python packages on Databricks cluster...")

        try:
            self.__submit_python_script_on_job_cluster(dbfs_script_path, package_metadata, wait=True)

        except BaseException:
            self.__logger.error("Build failed")
            return

        finally:
            self.__dbfs_file_uploader.upload(requirements_txt_redacted.encode("utf-8"), str(dbfs_requirements_path), overwrite=True)

        self.__logger.info("Build done")

        self.__logger.info(f"Downloading built packages to {local_dependencies_dir}...")
        self.__dbfs_file_downloader.download_directory(dbfs_dependencies_dir, local_dependencies_dir, overwrite=True)
        self.__logger.info("Download done, set dbxdeploy.target.package.offline_install: True in config and deploy")

    def __submit_python_script_on_job_cluster(
        self,
        script_path: PurePosixPath,
        package_metadata: PackageMetadata,
        wait: bool = False,
    ):
        submited_run = self.__dbx_api.jobs.submit_run(
            run_name=package_metadata.get_job_run_name(),
            new_cluster=self.__job_cluster_definition,
            spark_python_task=dict(python_file=str(script_path)),
        )

        run = self.__dbx_api.jobs.get_run(run_id=submited_run["run_id"])

        if wait:
            while run["state"]["life_cycle_state"] not in ["TERMINATED", "SKIPPED", "INTERNAL_ERROR"]:
                time.sleep(15)
                run = self.__dbx_api.jobs.get_run(run_id=submited_run["run_id"])

            return self.__dbx_api.jobs.get_run_output(submited_run["run_id"])

        return run



import sys
import re
from typing import List
from pathlib import Path
from dbxdeploy.poetry.PoetryPathResolver import PoetryPathResolver
from dbxdeploy.package.RequirementsConfig import RequirementsConfig
from dbxdeploy.shell.runner import run_and_read_output
from dbxdeploy.package.requirements_regex import LOCAL_FILE_LINE_REGEX


class RequirementsGenerator:
    def __init__(
        self,
        project_base_dir: Path,
        poetry_path_resolver: PoetryPathResolver,
    ):
        self.__project_base_dir = project_base_dir
        self.__poetry_path_resolver = poetry_path_resolver
        self.__python_executable = sys.executable

    def generate(self, requirements_config: RequirementsConfig) -> str:
        options = ["--format requirements.txt", "--without-hashes"]

        if requirements_config.should_include_dev_dependencies:
            options.append("--dev")

        if requirements_config.should_include_credentials:
            options.append("--with-credentials")

        requirements_txt = self.__generate_requirements_txt(options)

        if requirements_config.should_exclude_index_info:
            requirements_txt = self.__remove_index_info(requirements_txt)

        if requirements_config.should_exclude_file_dependencies:
            requirements_txt = self.__remove_file_dependencies(requirements_txt)

        if requirements_config.should_redact_credentials:
            requirements_txt = self.__redact_credentials(requirements_txt)

        return requirements_txt

    def __generate_requirements_txt(self, options: List[str]) -> str:
        command = f"{self.__python_executable} {self.__poetry_path_resolver.get_poetry_path()} export {' '.join(options)}"

        output = run_and_read_output(command, cwd=str(self.__project_base_dir), shell=True)

        return output

    def __remove_index_info(self, requirements_txt: str) -> str:
        requirements = requirements_txt.splitlines()

        requirements = [
            requirement
            for requirement in requirements
            if not requirement.strip() == ""
            and not requirement.startswith("--index-url")
            and not requirement.startswith("--extra-index-url")
        ]

        return "\n".join(requirements)

    def __remove_file_dependencies(self, requirements_txt: str) -> str:
        requirements = requirements_txt.splitlines()

        requirements = [requirement for requirement in requirements if not re.match(LOCAL_FILE_LINE_REGEX, requirement)]

        return "\n".join(requirements)

    def __redact_credentials(self, requirements_txt: str) -> str:
        requirements = requirements_txt.splitlines()

        for index, line in enumerate(requirements):
            if line.startswith("--index-url") and "@" in line or line.startswith("--extra-index-url") and "@" in line:
                matches = re.match(r"(.*http://|.*https://)(.*?)(@.*)", line)
                redacted_line = matches.group(1) + "[REDACTED]" + matches.group(3)
                requirements[index] = redacted_line

        return "\n".join(requirements)



import unittest
from pyfonycore.bootstrap import bootstrapped_container
from dbxdeploy.dbc.CommandConverter import CommandConverter


class CommandConverterTest(unittest.TestCase):
    def setUp(self):
        container = bootstrapped_container.init("test")

        self.__command_converter = container.get(CommandConverter)  # type: CommandConverter

    def test_python_code(self):
        self.__test_command(
            'print("Hello world")',
            'print("Hello world")',
        )

    def test_python_code_multi_line_with_title(self):
        self.__test_command(
            'print("Hello world")\n\na = 5',
            '# DBTITLE 1,My Command Title\nprint("Hello world")\n\na = 5',
            showCommandTitle=True,
            commandTitle="My Command Title",
        )

    def test_magic(self):
        self.__test_command("%sql SELECT * FROM test_table", "# MAGIC %sql SELECT * FROM test_table")

    def test_magic_multi_line_with_title(self):
        self.__test_command(
            "%sql\n\nSELECT * FROM test_table",
            "# DBTITLE 1,My Command Title\n# MAGIC %sql\n# MAGIC \n# MAGIC SELECT * FROM test_table",
            showCommandTitle=True,
            commandTitle="My Command Title",
        )

    def test_install_master_package_whl(self):
        self.__test_command(
            "# %install_master_package_whl\nimport IPython, os\nIPython.get_ipython().run_line_magic('pip', f'install /dbfs/FileStore/jars/myproject/2020-03-21_08-56-14_fhpxgwblvi/myproject-1.0-py3-none-any.whl')",
            "# MAGIC %install_master_package_whl",
        )

    def test_install_master_package_whl_with_title(self):
        self.__test_command(
            "# %install_master_package_whl\nimport IPython, os\nIPython.get_ipython().run_line_magic('pip', f'install /dbfs/FileStore/jars/myproject/2020-03-21_08-56-14_fhpxgwblvi/myproject-1.0-py3-none-any.whl')",
            "# DBTITLE 0,My Command Title\n# MAGIC %install_master_package_whl",
            showCommandTitle=False,
            commandTitle="My Command Title",
        )

    def __test_command(self, command_code: str, expected_line: str, **kwargs):
        command = {**{"command": command_code, "position": 1, "commandTitle": ""}, **kwargs}

        result = self.__command_converter.convert(command)

        self.assertEqual(result, expected_line)


if __name__ == "__main__":
    unittest.main()



import tomlkit
from tomlkit import table
from tomlkit.toml_document import TOMLDocument
from pathlib import Path
from dbxdeploy.package.RequirementsLineConverter import RequirementsLineConverter
from dbxdeploy.package.RequirementsGenerator import RequirementsGenerator
from dbxdeploy.package.RequirementsConfig import RequirementsConfig
from dbxdeploy.package.PackageMetadata import PackageMetadata


class LockedPyprojectCreator:
    def __init__(
        self,
        requirements_line_converter: RequirementsLineConverter,
        requirements_generator: RequirementsGenerator,
    ):
        self.__requirements_line_converter = requirements_line_converter
        self.__requirements_generator = requirements_generator

    def create(self, package_metadata: PackageMetadata, pyproject_orig_path: Path, pyproject_new_path: Path):
        toml_doc = self.get_locked_pyproject_toml(package_metadata, pyproject_orig_path)

        with pyproject_new_path.open("w") as t:
            t.write(toml_doc.as_string())

    def get_locked_pyproject_toml(self, package_metadata: PackageMetadata, pyproject_orig_path: Path) -> TOMLDocument:
        main_dependencies = self.__load_main_dependencies()
        toml_doc = self.__generate_pyproject_new(package_metadata, pyproject_orig_path, main_dependencies)

        return toml_doc

    def __load_main_dependencies(self) -> list:
        requirements_config = RequirementsConfig()
        requirements_config.exclude_index_info()
        requirements_txt = self.__requirements_generator.generate(requirements_config)

        return list(map(self.__requirements_line_converter.parse, requirements_txt.splitlines()))

    def __generate_pyproject_new(self, package_metadata: PackageMetadata, pyproject_orig_path: Path, requirements: list) -> TOMLDocument:
        with pyproject_orig_path.open("r") as t:
            toml_doc = tomlkit.parse(t.read())

            dependencies = toml_doc["tool"]["poetry"]["dependencies"]

            if "python" not in dependencies:
                raise Exception('"python" must be defined in [tool.poetry.dependencies]')

            new_dependencies = table()
            new_dependencies.add("python", dependencies["python"])

            for requirement in requirements:
                if self.__is_linux_dependency(requirement):
                    new_dependencies.add(*requirement)

            toml_doc["tool"]["poetry"]["version"] = package_metadata.package_version
            toml_doc["tool"]["poetry"]["dependencies"] = new_dependencies
            del toml_doc["tool"]["poetry"]["dev-dependencies"]

        return toml_doc

    def __is_linux_dependency(self, requirement):
        markers_present = isinstance(requirement[1], dict) and "markers" in requirement[1]

        if not markers_present:
            return True

        platform_info_present = "sys_platform" in requirement[1]["markers"] or "platform_system" in requirement[1]["markers"]

        if not platform_info_present:
            return True

        is_linux_dependency = (
            'sys_platform == "linux"' in requirement[1]["markers"]
            or 'sys_platform == "linux2"' in requirement[1]["markers"]
            or 'platform_system == "Linux"' in requirement[1]["markers"]
        )

        return is_linux_dependency



from argparse import Namespace
from pathlib import Path
from dbxdeploy.package.PackageBuilder import PackageBuilder
from consolebundle.ConsoleCommand import ConsoleCommand
from dbxdeploy.package.PackageMetadataLoader import PackageMetadataLoader


class MasterPackageBuilder(ConsoleCommand):
    def __init__(
        self,
        package_metadata_loader: PackageMetadataLoader,
        package_builder: PackageBuilder,
    ):
        self.__package_metadata_loader = package_metadata_loader
        self.__package_builder = package_builder

    def get_command(self) -> str:
        return "dbx:build-master-package"

    def get_description(self):
        return "Build the master package"

    def run(self, input_args: Namespace):
        project_base_dir = Path.cwd()
        package_metadata = self.__package_metadata_loader.load(project_base_dir)

        self.__package_builder.build(project_base_dir, package_metadata)



import tomlkit
from tomlkit.items import Table
from typing import List
from pathlib import Path
from dbxdeploy.package.Dependency import Dependency
from dbxdeploy.package.RequirementsLineConverter import RequirementsLineConverter
from dbxdeploy.package.RequirementsGenerator import RequirementsGenerator
from dbxdeploy.package.RequirementsConfig import RequirementsConfig


class PackageDependencyLoader:
    def __init__(
        self,
        requirements_line_converter: RequirementsLineConverter,
        requirements_generator: RequirementsGenerator,
    ):
        self.__requirements_generator = requirements_generator
        self.__requirements_line_converter = requirements_line_converter

    def load(self, project_base_dir: Path) -> List[Dependency]:
        poetry_lock_path = project_base_dir.joinpath("poetry.lock")
        poetry_lock_dependencies = self.__load_poetry_lock_dependencies(poetry_lock_path)

        main_dependencies = self.__load_main_dependencies()
        dependencies = []

        for dependency in main_dependencies:
            dependency_name = dependency[0]
            dependencies.append(
                Dependency(dependency_name, self.__find_poetry_lock_version_by_name(poetry_lock_dependencies, dependency_name))
            )

        return dependencies

    def __load_main_dependencies(self) -> list:
        requirements_config = RequirementsConfig()
        requirements_config.exclude_index_info()
        requirements_txt = self.__requirements_generator.generate(requirements_config)

        return list(map(self.__requirements_line_converter.parse, requirements_txt.splitlines()))

    def __load_poetry_lock_dependencies(self, lockfile_path: Path) -> List[Table]:
        with lockfile_path.open("r") as f:
            config = tomlkit.parse(f.read())

        return [package for package in config["package"] if package["category"] == "main" and package["name"]]

    def __find_poetry_lock_version_by_name(self, dependencies: List[Table], dependency_name: str) -> str:
        for dependency in dependencies:
            if dependency["name"].lower() == dependency_name.lower():
                return dependency["version"]

        raise Exception(f"Dependency {dependency_name} not found in poetry.lock")



import subprocess
from box import Box
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from dbxdeploy.package.PackageMetadata import PackageMetadata
from pathlib import PurePosixPath
from logging import Logger


class JobSubmitter:
    def __init__(
        self,
        workspace_base_dir: PurePosixPath,
        browser_config: Box,
        logger: Logger,
        dbx_api: DatabricksClient,
    ):
        self.__workspace_base_dir = workspace_base_dir
        self.__browser_config = browser_config
        self.__logger = logger
        self.__dbx_api = dbx_api

    def submit(self, notebook_path: PurePosixPath, cluster_id: str, package_metadata: PackageMetadata):
        notebook_release_path = self.__workspace_base_dir.joinpath(notebook_path)

        self.__logger.info(f"Submitting job for {notebook_release_path} to cluster {cluster_id}")

        submited_run = self.__dbx_api.jobs.submit_run(
            run_name=package_metadata.get_job_run_name(),
            existing_cluster_id=cluster_id,
            notebook_task=dict(notebook_path=str(notebook_release_path)),
        )

        self.__logger.info(f'Job {str(submited_run["run_id"])} created')

        run = self.__dbx_api.jobs.get_run(run_id=submited_run["run_id"])

        self.__open_job_in_databricks(run["run_page_url"])

    def __open_job_in_databricks(self, run_url: str):
        self.__logger.info(f"Opening {run_url}")

        arguments = [self.__browser_config.path]

        for argument in self.__browser_config.arguments:
            arguments.append(argument.replace("{run_url}", run_url))

        subprocess.run(arguments, check=True)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['dbxdeploy',
 'dbxdeploy.black',
 'dbxdeploy.cluster',
 'dbxdeploy.dbc',
 'dbxdeploy.dbfs',
 'dbxdeploy.deploy',
 'dbxdeploy.filesystem',
 'dbxdeploy.git',
 'dbxdeploy.job',
 'dbxdeploy.notebook',
 'dbxdeploy.notebook.converter',
 'dbxdeploy.package',
 'dbxdeploy.poetry',
 'dbxdeploy.repos',
 'dbxdeploy.shell',
 'dbxdeploy.string',
 'dbxdeploy.utils',
 'dbxdeploy.workspace']

package_data = \
{'': ['*'], 'dbxdeploy': ['_config/*']}

install_requires = \
['Jinja2>=2.0.0,<3.0.0',
 'console-bundle>=0.5,<0.6',
 'databricks-cli>=0.16.0,<0.17.0',
 'dbx-notebook-exporter>=0.4.0,<0.5.0',
 'nbconvert>=5.6.0,<6.0.0',
 'pyfony-bundles>=0.4.0,<0.5.0',
 'pyfony-core>=0.8.0,<0.9.0',
 'pygit2>=1.3,<2.0',
 'python-box>=3.4,<4.0',
 'tomlkit>=0.5.8,<1.0.0']

entry_points = \
{'pyfony.bundle': ['create = dbxdeploy.DbxDeployBundle:DbxDeployBundle']}

setup_kwargs = {
    'name': 'dbx-deploy',
    'version': '1.2.0.dev4',
    'description': 'Databricks Deployment Tool',
    'long_description': '# DBX deploy bundle\n\nThis bundle allows you to deploy notebooks along with custom code to Databricks. It is part of the [Daipe Framework](https://www.daipe.ai).  \n\n## Resources\n\n* [Documentation](https://docs.daipe.ai/deploy-demo-project/)\n',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/daipe-ai/dbx-deploy',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from base64 import b64encode
from pathlib import PurePosixPath
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from requests.exceptions import HTTPError


class DbcUploader:
    def __init__(self, dbx_api: DatabricksClient):
        self.__dbx_api = dbx_api

    def upload(self, dbc_content: bytes, release_path: PurePosixPath):
        content_to_upload = b64encode(dbc_content).decode()

        try:
            self.__perform_upload(content_to_upload, release_path)
        except HTTPError:
            self.__dbx_api.workspace.mkdirs(str(release_path.parent))
            self.__perform_upload(content_to_upload, release_path)

    def __perform_upload(self, content_to_upload, release_path: PurePosixPath):
        self.__dbx_api.workspace.import_workspace(
            str(release_path),
            format="DBC",
            content=content_to_upload,
        )



import unittest
from pyfonycore.bootstrap import bootstrapped_container
from dbxdeploy.dbc.CommandConverter import CommandConverter
from dbxdeploy.dbc.CommandsConverter import CommandsConverter
from dbxdeploy.notebook.converter.CommandSeparatorConverter import CommandSeparatorConverter
from dbxdeploy.black.BlackChecker import BlackChecker
from logging import Logger


class CommandsConverterTest(unittest.TestCase):
    def setUp(self):
        container = bootstrapped_container.init("test")
        self.__logger = Logger("test")
        self.__command_converter = container.get(CommandConverter)  # type: CommandConverter

    def test_forced_end_file_new_line(self):
        result = self.__create_result(True, False)

        self.__assert_notebook(
            [
                "# Databricks notebook source",
                "# MAGIC %run /foo/bar",
                "",
                "# COMMAND ----------",
                "",
                'print("Hello world")',
                "",
            ],
            result,
        )

    def test_no_forced_end_file_new_line(self):
        result = self.__create_result(False, False)

        self.__assert_notebook(
            [
                "# Databricks notebook source",
                "# MAGIC %run /foo/bar",
                "",
                "# COMMAND ----------",
                "",
                'print("Hello world")',
            ],
            result,
        )

    def __create_result(self, force_end_file_newline: bool, black_enabled: bool):
        commands_converter = CommandsConverter(
            force_end_file_newline, self.__logger, self.__command_converter, BlackChecker(black_enabled)
        )  # type: CommandsConverter

        commands = [
            {"command": 'print("Hello world")', "position": 2, "commandTitle": "", "subtype": "command"},
            {"command": "%run /foo/bar", "position": 1, "commandTitle": "", "subtype": "command"},
            {"command": "", "position": 1.33, "commandTitle": "", "subtype": "command"},
            {"command": "", "position": 1.66, "commandTitle": "", "subtype": "command"},
        ]

        return commands_converter.convert(
            commands,
            CommandSeparatorConverter.first_line,
            CommandSeparatorConverter.cell_separator,
        )

    def __assert_notebook(self, expected: list, result_code: str):
        result_lines = result_code.split("\n")

        self.assertEqual(expected, result_lines)


if __name__ == "__main__":
    unittest.main()



import os
from argparse import Namespace, ArgumentParser
from logging import Logger
from consolebundle.ConsoleCommand import ConsoleCommand
from consolebundle.StrToBool import str2_bool
from databricks_cli.dbfs.api import DbfsApi
from databricks_cli.dbfs.dbfs_path import DbfsPath


class DbfsUploadCommand(ConsoleCommand):
    def __init__(
        self,
        logger: Logger,
        dbfs_api: DbfsApi,
    ):
        self.__logger = logger
        self.__dbfs_api = dbfs_api

    def get_command(self) -> str:
        return "dbx:dbfs:upload"

    def get_description(self):
        return "Uploads file to DBFS"

    def configure(self, argument_parser: ArgumentParser):
        argument_parser.add_argument(dest="source_file_path", help="Source file path")
        argument_parser.add_argument(dest="target_file_path", help="Target file path")
        argument_parser.add_argument(
            "--overwrite", dest="overwrite", type=str2_bool, nargs="?", const=True, default=False, help="Overwrite target file"
        )

    def run(self, input_args: Namespace):
        if os.path.isabs(input_args.source_file_path):
            source_file_path = input_args.source_file_path
        else:
            source_file_path = os.getcwd() + os.sep + input_args.source_file_path

        self.__logger.info(f"Uploading {source_file_path} to {input_args.target_file_path}")

        self.__dbfs_api.put_file(
            source_file_path,
            DbfsPath(input_args.target_file_path),
            input_args.overwrite,
        )

        self.__logger.info("File successfully uploaded")



from argparse import Namespace, ArgumentParser
from pathlib import Path
from dbxdeploy.package.DependencyBuilder import DependencyBuilder
from consolebundle.ConsoleCommand import ConsoleCommand
from dbxdeploy.package.PackageMetadataLoader import PackageMetadataLoader


class DependencyBuilderCommand(ConsoleCommand):
    def __init__(
        self,
        project_base_dir: Path,
        package_metadata_loader: PackageMetadataLoader,
        dependency_builder: DependencyBuilder,
    ):
        self.__project_base_dir = project_base_dir
        self.__package_metadata_loader = package_metadata_loader
        self.__dependency_builder = dependency_builder

    def configure(self, argument_parser: ArgumentParser):
        argument_parser.add_argument("--dev", action="store_true")

    def get_command(self) -> str:
        return "dbx:build-dependencies"

    def get_description(self):
        return "Build linux dependencies on remote DBX cluster and then download them in local dependencies directory"

    def run(self, input_args: Namespace):
        package_metadata = self.__package_metadata_loader.load(self.__project_base_dir)

        self.__dependency_builder.build(self.__project_base_dir, package_metadata, dev_dependencies=input_args.dev)



import io
import os
import subprocess


def run_and_read_output(command: str, cwd: str = os.getcwd(), shell=False):
    proc = subprocess.Popen(command, cwd=cwd, stdout=subprocess.PIPE, shell=shell)
    output = io.TextIOWrapper(proc.stdout, encoding="utf-8").read().rstrip()

    if not output:
        raise Exception(f"No output returned to stdout for: {command}")

    return output


def run_shell_command(command: str, cwd: str = os.getcwd(), shell=False):
    proc = subprocess.Popen(command, cwd=cwd, shell=shell)
    proc.communicate()

    if proc.returncode > 0:
        raise Exception(f"Shell command failed with code: {proc.returncode}")



import os
from dbxdeploy.dbc.CommandConverter import CommandConverter
from dbxdeploy.black.BlackChecker import BlackChecker
from logging import Logger


class CommandsConverter:
    def __init__(self, force_end_file_newline: bool, logger: Logger, command_converter: CommandConverter, black_checker: BlackChecker):
        self.__force_end_file_newline = force_end_file_newline
        self.__logger = logger
        self.__command_converter = command_converter
        self.__black_checker = black_checker

    def convert(self, commands: list, first_line: str, cell_separator: str) -> str:
        while commands[len(commands) - 1]["command"] == "":
            commands.pop()

        commands.sort(key=lambda command: command["position"])
        commands = [command for command in commands if command["subtype"] == "command"]
        commands = list(map(self.__command_converter.convert, commands))

        output = f"\n\n{cell_separator}\n\n".join(commands)

        return self.__format(first_line, output)

    def __format(self, first_line: str, output: str):
        def format_without_black(starting_line: str, text: str):
            if self.__force_end_file_newline and text[-1:] != "\n":
                text += "\n"

            return starting_line + "\n" + text

        if self.__black_checker.is_black_enabled and self.__black_checker.is_black_installed:
            import black

            black_config = black.parse_pyproject_toml(os.getcwd() + "/pyproject.toml")
            black_mode = black.Mode(**black_config)

            return black.format_str(first_line + "\n" + output, mode=black_mode)

        else:
            return format_without_black(first_line, output)



from pathlib import Path, PurePosixPath


class Notebook:
    def __init__(
        self,
        path: Path,
        relative_path: Path,
        databricks_relative_path: PurePosixPath,
    ):
        self.__path = path
        self.__relative_path = relative_path
        self.__databricks_relative_path = databricks_relative_path

    @property
    def path(self) -> Path:
        return self.__path

    @property
    def relative_path(self) -> Path:
        return self.__relative_path

    @property
    def databricks_relative_path(self) -> PurePosixPath:
        return self.__databricks_relative_path



from pygit2 import GitError
from pathlib import PurePosixPath
from dbxdeploy.git.CurrentBranchResolver import CurrentBranchResolver
from dbxdeploy.package.PackageMetadata import PackageMetadata


class TargetPathsResolver:
    def __init__(
        self,
        package_deploy_path: str,
        package_release_path: str,
        dependencies_deploy_path: str,
        dependencies_release_path: str,
        workspace_release_path: str,
        workspace_current_path: str,
        current_branch_resolver: CurrentBranchResolver,
    ):
        self.__package_deploy_path = package_deploy_path
        self.__package_release_path = package_release_path
        self.__dependencies_deploy_path = dependencies_deploy_path
        self.__dependencies_release_path = dependencies_release_path
        self.__workspace_release_path = PurePosixPath(workspace_release_path)
        self.__workspace_current_path = PurePosixPath(workspace_current_path)
        self.__current_branch_resolver = current_branch_resolver

    def get_package_upload_path_for_deploy(self, package_metadata: PackageMetadata):
        return self.__replace_package_path(package_metadata, self.__package_deploy_path)

    def get_package_upload_path_for_release(self, package_metadata: PackageMetadata):
        return self.__replace_package_path(package_metadata, self.__package_release_path)

    def get_dependency_upload_path_for_deploy(self, package_metadata: PackageMetadata, dependency_filename):
        return self.__replace_dependency_path(package_metadata, self.__dependencies_deploy_path, dependency_filename)

    def get_dependency_upload_path_for_release(self, package_metadata: PackageMetadata, dependency_filename):
        return self.__replace_dependency_path(package_metadata, self.__dependencies_release_path, dependency_filename)

    def get_dependencies_upload_dir_for_deploy(self, package_metadata: PackageMetadata):
        return self.__replace_path(package_metadata, self.__dependencies_deploy_path.rstrip("{package_filename}"), None)

    def get_dependencies_upload_dir_for_release(self, package_metadata: PackageMetadata):
        return self.__replace_path(package_metadata, self.__dependencies_release_path.rstrip("{package_filename}"), None)

    def get_workspace_release_path(self, package_metadata: PackageMetadata) -> PurePosixPath:
        return self.__replace_workspace_path(package_metadata, self.__workspace_release_path)

    def get_workspace_current_path(self, package_metadata: PackageMetadata) -> PurePosixPath:
        return self.__replace_workspace_path(package_metadata, self.__workspace_current_path)

    def __replace_package_path(self, package_metadata: PackageMetadata, package_path: str):
        return self.__replace_path(package_metadata, package_path, package_metadata.get_package_filename())

    def __replace_dependency_path(self, package_metadata: PackageMetadata, package_path: str, dependency_filename):
        return self.__replace_path(package_metadata, package_path, dependency_filename)

    def __replace_path(self, package_metadata: PackageMetadata, package_path: str, package_filename: str):
        replacements = {
            "package_name": package_metadata.package_name,
            "package_filename": package_filename,
            "current_time": package_metadata.date_time.strftime("%Y-%m-%d_%H-%M-%S"),
            "random_string": package_metadata.random_string,
        }

        if "{current_branch}" in package_path:
            try:
                replacements["current_branch"] = self.__current_branch_resolver.resolve()
            except GitError:
                replacements["current_branch"] = "__no_git_repo__"

        return package_path.format(**replacements)

    def __replace_workspace_path(self, package_metadata: PackageMetadata, workspace_path: PurePosixPath):
        replacements = {
            "current_time": package_metadata.date_time.strftime("%Y-%m-%d_%H:%M:%S"),
            "random_string": package_metadata.random_string,
        }

        if "{current_branch}" in str(workspace_path):
            replacements["current_branch"] = self.__current_branch_resolver.resolve()

        return PurePosixPath(str(workspace_path).format(**replacements))



import re
from logging import Logger
from typing import List
from zipfile import ZipInfo, ZipFile
from dbxdeploy.utils.DatabricksClient import DatabricksClient
from pathlib import PurePosixPath
from pygit2 import GitError
from dbxdeploy.git.CurrentBranchResolver import CurrentBranchResolver
from dbxdeploy.notebook.converter.NotebookConverterInterface import NotebookConverterInterface
from dbxdeploy.notebook.converter.UnexpectedSourceException import UnexpectedSourceException
from dbxdeploy.notebook.loader import load_notebook
from dbxdeploy.workspace.DbcFilesHandler import DbcFilesHandler
from dbxdeploy.workspace.WorkspaceExportException import WorkspaceExportException
from dbxdeploy.workspace.WorkspaceImporter import WorkspaceImporter
from dbxdeploy.workspace.WorkspaceExporter import WorkspaceExporter
from dbxdeploy.notebook.Notebook import Notebook


class CurrentDirectoryUpdater:
    def __init__(
        self,
        workspace_base_dir: PurePosixPath,
        git_dev_branch: str,
        logger: Logger,
        dbx_api: DatabricksClient,
        notebook_converter: NotebookConverterInterface,
        workspace_exporter: WorkspaceExporter,
        dbc_files_handler: DbcFilesHandler,
        workspace_importer: WorkspaceImporter,
        current_branch_resolver: CurrentBranchResolver,
    ):
        self.__workspace_base_dir = workspace_base_dir
        self.__git_dev_branch = git_dev_branch
        self.__logger = logger
        self.__dbx_api = dbx_api
        self.__notebook_converter = notebook_converter
        self.__workspace_exporter = workspace_exporter
        self.__dbc_files_handler = dbc_files_handler
        self.__workspace_importer = workspace_importer
        self.__current_branch_resolver = current_branch_resolver

    def update(self, notebooks: List[Notebook], current_release_path: PurePosixPath, package_path: str, dependencies_dir_path: str):
        if self.__should_remove_missing_notebooks():
            self.__remove_missing_notebooks(current_release_path, notebooks)

        self.__update_notebooks(current_release_path, notebooks, package_path, dependencies_dir_path)

    def update_only_master_package_notebook(
        self, notebook: Notebook, current_release_path: PurePosixPath, package_path: str, dependencies_dir_path: str
    ):
        self.__update_notebooks(current_release_path, [notebook], package_path, dependencies_dir_path)

    def __remove_missing_notebooks(self, current_release_path: PurePosixPath, notebooks: List[Notebook]):
        existing_notebooks_full_paths = self.__resolve_existing_notebooks_paths(current_release_path)

        existing_notebooks = set(map(lambda path: re.sub(r"\.python$", "", path), existing_notebooks_full_paths))
        new_notebooks = set(map(lambda notebook: str(notebook.databricks_relative_path), notebooks))

        for notebook_to_delete in existing_notebooks - new_notebooks:
            full_notebook_path = self.__workspace_base_dir.joinpath(notebook_to_delete)
            self.__logger.warning("Removing deleted/missing notebook {}".format(full_notebook_path))
            self.__dbx_api.workspace.delete(str(full_notebook_path))

    def __update_notebooks(
        self, current_release_path: PurePosixPath, notebooks: List[Notebook], package_path: str, dependencies_dir_path: str
    ):
        for notebook in notebooks:
            target_path = current_release_path.joinpath(notebook.databricks_relative_path)
            source = load_notebook(notebook.path)

            try:
                self.__notebook_converter.validate_source(source)
            except UnexpectedSourceException:
                self.__logger.debug(f"Skipping unrecognized file {notebook.relative_path}")
                continue

            script = self.__notebook_converter.to_workspace_import_notebook(source, package_path, dependencies_dir_path)

            self.__logger.info("Updating {}".format(target_path))
            self.__workspace_importer.overwrite_script(script, target_path)

    def __should_remove_missing_notebooks(self):
        try:
            current_git_branch = self.__current_branch_resolver.resolve()
        except GitError:
            return False

        return current_git_branch == self.__git_dev_branch

    def __resolve_existing_notebooks_paths(self, current_release_path: PurePosixPath):
        file_names = []

        def resolve_filenames(zip_file: ZipFile, file: ZipInfo):
            if file.orig_filename[-1:] == "/":
                return

            """
            _current/myproject/foo/bar.python -> myproject/foo/bar.python (dbx:release)
            mybranch/myproject/foo/bar.python -> myproject/foo/bar.python (dbx:deploy)
            """
            file_path_without_rootdir = file.orig_filename[file.orig_filename.index("/") + 1 :]  # noqa: 5203

            file_names.append(file_path_without_rootdir)

        try:
            dbc_content = self.__workspace_exporter.export(current_release_path)
        except WorkspaceExportException:
            self.__logger.error("Unable to compare new notebooks to the existing ones in workspace")
            return []

        self.__dbc_files_handler.handle(dbc_content, resolve_filenames)

        return file_names



from pathlib import Path


class PoetryPathResolver:
    def __init__(self, poetry_path: str):
        self.__poetry_path = poetry_path

    def get_poetry_path(self) -> Path:
        return Path(self.__poetry_path).expanduser()



from pathlib import Path
import tomlkit
from datetime import datetime
from dbxdeploy.package.PackageMetadata import PackageMetadata
from dbxdeploy.package.PackageDependencyLoader import PackageDependencyLoader
from dbxdeploy.string.RandomStringGenerator import RandomStringGenerator


class PackageMetadataLoader:
    def __init__(
        self,
        package_dependency_loader: PackageDependencyLoader,
    ):
        self.__package_dependency_loader = package_dependency_loader

    def load(self, project_base_dir: Path) -> PackageMetadata:
        pyproject_path = project_base_dir.joinpath("pyproject.toml")

        with pyproject_path.open("r") as t:
            lock = tomlkit.parse(t.read())

            tool_params = lock["tool"]["poetry"]

            package_name = str(tool_params["name"])
            package_version = str(tool_params["version"])

            return PackageMetadata(
                package_name,
                package_version,
                datetime.now(),
                RandomStringGenerator().generate(10),
                self.__package_dependency_loader.load(project_base_dir),
            )
